#include "bjack.h"
#include "dialogs.h"
#include "sman.h"
#include <sman.rsg>

/*************************************************************
*
* Logging class
*
**************************************************************/

CLogger::CLogger()
{
	logFilename.Copy(CEikonEnv::Static()->EikAppUi()->Application()->AppFullName());
	logFilename.SetLength(logFilename.Length() - 3);
	logFilename.Append(_L("log.txt"));
}

CLogger::~CLogger()
{
}

void CLogger::updateLog(const TDesC *dialogLine, const TInt numExtraLines, TUint logType)
{
	CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
	if ((tempUiObj->iTaskView->configData.logFlags & logType) == 0)
		return;

	TBuf<MAX_LOG_LINE_SIZE * MAX_LOG_LINES> tempLine;
	TTime tTime;

	tTime.HomeTime();
	tTime.FormatL(tempLine, _L("%/0%1%/1%2%/2%3%/3 %-B%:0%J%:1%T%:2%S%:3%+B  "));
	tempLine.Append(*dialogLine);
	
	if (*localAutoSave)
	{
		TBuf<MAX_LOG_LINE_SIZE * MAX_LOG_LINES> fileOutput;
		
		fileOutput.Copy(tempLine);
		fileOutput.Append(_L("\r\n"));
		for (int i = 0; i < numExtraLines;i ++)
			fileOutput.Append(_L("\r\n"));
		if (!saveLog(EFalse, &fileOutput))
			CEikonEnv::Static()->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVEBLUELOG);
	}
	tempLine.Append(CEditableText::ELineBreak);
	tempLine.Append(CEditableText::ELineBreak);

	// Are we overflowing the buffer?
	if ((logBuffer->Length() + tempLine.Length()) >= (MAX_LOG_LINES * MAX_LOG_LINE_SIZE))
	{
		// Yes. Delete the older characters
		logBuffer->Delete(0, tempLine.Length());
	}
	logBuffer->Append(tempLine);
	if (logDialog != NULL)
	{
		CEikGlobalTextEditor *logEditor;
		logEditor = STATIC_CAST(CEikGlobalTextEditor*, logDialog->Control(controlId));
		logEditor->SetTextL(logBuffer);
		logEditor->SetCursorPosL(logEditor->TextLength(), 0);
		logEditor->DrawNow();
	}
}

TBool CLogger::saveLog(TBool overwrite, TDesC *buffer)
{
	RFile outStream;
	TBuf8<((MAX_LOG_LINES + 1) * MAX_LOG_LINE_SIZE) * 2> buffer8bit;
	TInt openResult;

	// Use copy to convert from unicode to non-unicode
	buffer8bit.Copy(*buffer);

	if (overwrite)
		openResult = outStream.Replace(CEikonEnv::Static()->FsSession(), logFilename, EFileWrite);
	else
	{
		if ((openResult = outStream.Open(CEikonEnv::Static()->FsSession(), logFilename, EFileWrite)) != KErrNone)
			openResult = outStream.Replace(CEikonEnv::Static()->FsSession(), logFilename, EFileWrite);
	}
		
	if (openResult == KErrNone)
	{
		TInt aPos;
		
		aPos = 0;
		outStream.Seek(ESeekEnd, aPos);
		outStream.Write(buffer8bit);
		outStream.Close();
		return ETrue;
	}
	else
		return EFalse;
}

/*************************************************************
*
* Scanner class
*
**************************************************************/

CScanner::~CScanner()
{
	if (hostResolverOpen)
	{
		hostResolver.Close();
		hostResolverOpen = EFalse;
	}
	if (socketServOpen)
	{
		socketServ.Close();
		socketServOpen = EFalse;
	}
	delete logHelper;
}

CScanner::CScanner(TFileName *appPath) : CTimer(EPriorityNormal)
{	
	vCardNames[0].Copy(*appPath);
	vCardNames[0].Append(_L("SMAN.VCF"));

	scanStarted = EFalse;
	scannerStatus = isIdle;
	hostResolverOpen = EFalse;
	socketServOpen = EFalse;
	
	logHelper = new (ELeave) CLogger;
	logHelper->logBuffer = &logBuffer;
	logHelper->controlId = cBluejackLog;

	socketServ.Connect();
	socketServOpen = ETrue;
	// L2CAP, SDP, RFCOMM, BTLinkManager
	if ((socketServ.FindProtocol(_L("BTLinkManager"), protoDesc)) != KErrNone)
	{
		socketServ.Close();
		socketServOpen = EFalse;
		User::InfoPrint(_L("BT Link Manager missing!"));
		User::Leave(0);
	}
	CTimer::ConstructL();
	CActiveScheduler::Add(this);
}

void CScanner::doRealScan(void)
{
	_LIT(txt, "Scanning...");
	logHelper->updateLog(&txt, 0, CLogger::logScanning);

	// Check if user switched off bluetooth
	if (hostResolverOpen)
		hostResolver.Close(); //++++++++++++++++++
	if (hostResolver.Open(socketServ, protoDesc.iAddrFamily, protoDesc.iProtocol) != KErrNone)
	{
		hostResolverOpen = EFalse;
		User::InfoPrint(_L("Bluetooth not enabled!"));
		_LIT(txt, "Bluetooth not enabled");
		logHelper->updateLog(&txt, 0, CLogger::logAll);
		scanStarted = EFalse;
		scannerStatus = isIdle;
		return;
	}
	hostResolverOpen = ETrue;
	if (obexFile)
	{
		delete obexFile;
		obexFile = NULL;
	}
	if (obexClient)
	{
		delete obexClient;
		obexClient = NULL;
	}
    obexFile = CObexFileObject::NewL(TPtrC(NULL, 0));
	TInt err;
	TRAP(err, obexFile->InitFromFileL(vCardNames[0]));
	if (err)
	{
		_LIT(txt, "Couldn't open vcard");
		logHelper->updateLog(&txt, 0, CLogger::logAll);
		stopScan();
	}
	else
	{
		// Start the discovery inquiry
		sockAddr.SetIAC(KGIAC);
		sockAddr.SetAction(resolveAction);
		iStatus = KRequestPending;
		scannerStatus = isInquiring;
		hostResolver.GetByAddress(sockAddr, nameEntry, iStatus);
		SetActive();
	}
}

void CScanner::timeoutAbort(void)
{
	if (obexClient)
	{
		//obexClient->Abort();
		delete obexClient;
		obexClient = NULL;
	}

	if (IsActive())
		Cancel();

	if (scannerStatus != isShutdown)
	{
		startRestTime.HomeTime();
		currentNoDevicePause = STACK_COOLOFF_COUNTDOWN;
		firstRest = ETrue;
		rest();
	}
	else
		stopScan();
}

void CScanner::startScan(TBool *autoSave, TBool resolveName, TBool *showSysMsg, TBool *proxSweepOnce, TBool *noDeviceAttempts, TBool *noDevicePause)
{
	logHelper->localAutoSave = autoSave;
	localShowSysMsg = showSysMsg;
	localProxSweepOnce = proxSweepOnce;
	localNoDeviceAttempts = noDeviceAttempts;
	localNoDevicePause = noDevicePause;
	currentNoDeviceAttempts = 0;
	currentDeviceIndex = MAX_BTDEV_ADDR;
	for (int i = 0 ; i < MAX_BTDEV_ADDR; i++)
	{
		btDevice[i].isBluejacked = 0;
		btDevice[i].btDevAddr[0] = 0;
		btDevice[i].btDevAddr[1] = 0;
		btDevice[i].btDevAddr[2] = 0;
		btDevice[i].btDevAddr[3] = 0;
		btDevice[i].btDevAddr[4] = 0;
		btDevice[i].btDevAddr[5] = 0;
		btDevice[i].obexPort = -1;										
	}
	if (resolveName)
		resolveAction = KHostResInquiry + KHostResName;
	else
		resolveAction = KHostResInquiry;
	scanStarted = ETrue;
	User::InfoPrint(_L("Scanner started."));
	doRealScan();
}

void CScanner::stopScan(void)
{
	if (IsActive())
	{
		if (scannerStatus == isRest)
		{
			Cancel();
			After(1);
		}
		scannerStatus = isShutdown;
		User::InfoPrint(_L("Terminating scanner. Wait."));
	}
	else
	{
		timeoutCounter.stopTimer();
		if (obexFile)
		{
			delete obexFile;
			obexFile = NULL;
		}
		if (obexClient)
		{
			delete obexClient;
			obexClient = NULL;
		}
		
		scannerStatus = isIdle;
		scanStarted = EFalse;
		if (hostResolverOpen)
		{
			hostResolver.Close();
			hostResolverOpen = EFalse;
		}
		_LIT(txt, "Scanner terminated");
		logHelper->updateLog(&txt, 2, CLogger::logScanning);
		User::InfoPrint(_L("Scanner terminated"));
	}
}

void CScanner::logWithLookup(TPtrC theText, TUint logType)
{	
	if (resolveAction == KHostResInquiry)
	{
		TBuf<70> temp;
		temp.Format(_L("%x:%x:%x:%x:%x:%x - "), btDevice[currentDeviceIndex].btDevAddr[0], 
			btDevice[currentDeviceIndex].btDevAddr[1], btDevice[currentDeviceIndex].btDevAddr[2], 
			btDevice[currentDeviceIndex].btDevAddr[3], btDevice[currentDeviceIndex].btDevAddr[4], 
			btDevice[currentDeviceIndex].btDevAddr[5]);
		temp.Append(theText);
		logHelper->updateLog(&temp, 0, logType);
	}
	else
	{
		TBuf<sizeof(THostName) + 40> temp;
		temp.Copy(btDevice[currentDeviceIndex].btNames);
		temp.Append(_L(" - "));
		temp.Append(theText);
		logHelper->updateLog(&temp, 0, logType);
	}
}

void CScanner::rest(void)
{
	if (firstRest)
	{
		_LIT(txt, "Stack cool off");
		logHelper->updateLog(&txt, 0, CLogger::logDecays);
		scannerStatus = isRest;
		firstRest = EFalse;
	}
	After(TIMER_GRANULARITY);
}

void CScanner::bluejackNextDevice(void)
{
	if (getNextBTDeviceIndex())
	{
		if (obexClient)
		{
			delete obexClient;
			obexClient = NULL;
		}
		iStatus = KRequestPending;
		if (btDevice[currentDeviceIndex].obexPort == -1)
		{
			scannerStatus = isOBEXSearch;
			obexRetries = OBEX_RETRIES;
			obexSearcher.start(&iStatus, &(btDevice[currentDeviceIndex].btDevAddr));
			SetActive();
		}
		else
		{
			// This is a hack... :(
			obexSearcher.RFCOMMPort = btDevice[currentDeviceIndex].obexPort;
			scannerStatus = isOBEXSearch;
			RunL();
		}
	}
	else
	{
		currentNoDeviceAttempts++;
		if (*localProxSweepOnce == EFalse)
		{
			if (currentNoDeviceAttempts > *localNoDeviceAttempts)
			{
				currentNoDeviceAttempts = 0;
				startRestTime.HomeTime();
				currentNoDevicePause = *localNoDevicePause;
				firstRest = ETrue;
				rest();
			}
			else
				doRealScan();
		}
		else
			stopScan();
	}
}

void CScanner::RunL()
{
	if (scanStarted)
	{
		CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
		if (scannerStatus == isRest)
		{
			TTime currentTime;
			TTimeIntervalSeconds interval;
			
			currentTime.HomeTime();
			if (currentTime.SecondsFrom(startRestTime, interval) != 0)
				interval = CONFIG_NO_DEVICE_PAUSE + 1;
			if (interval.Int() >= currentNoDevicePause)
				bluejackNextDevice();
			else
				rest();
		} 
		else if (scannerStatus == isOBEXDisconnect)
		{
			timeoutCounter.stopTimer();
			startRestTime.HomeTime();
			currentNoDevicePause = STACK_COOLOFF_COUNTDOWN;
			firstRest = ETrue;
			rest();
		}
		else if (scannerStatus == isOBEXSend)
		{
			timeoutCounter.stopTimer();
			if (iStatus == KErrNone)
			{
				logWithLookup(_L("Bluejacked!"), CLogger::logBluejack);
				if (*localShowSysMsg)
					User::InfoPrint(_L("Bluejacked!"));
				// Update the device's status and timer
				btDevice[currentDeviceIndex].isBluejacked = 2;
				btDevice[currentDeviceIndex].lastBluejackedTime.HomeTime();
			}
			else
			{
				TBuf<30> errText;
				
				errText.Format(_L("Bluejack error %d"), iStatus);
				logWithLookup(errText, CLogger::logBluejack);
				
				// If the bluejack failed, marked as a cached item
				// and timestamp it
				btDevice[currentDeviceIndex].isBluejacked = 3;
				btDevice[currentDeviceIndex].lastBluejackedTime.HomeTime();
			}
			
			// Start the timeout counter
			timeoutCounter.startTimer(this, tempUiObj->iTaskView->configData.obexTimeout);
			
			// Disconnect and restart
			scannerStatus = isOBEXDisconnect;
			iStatus = KRequestPending;
			obexClient->Disconnect(iStatus);
			SetActive();
		}
		else if (scannerStatus == isOBEXConnect)
		{
			timeoutCounter.stopTimer();
			if (iStatus == KErrNone)
			{
				scannerStatus = isOBEXSend;
				
				// Start the timeout counter
				timeoutCounter.startTimer(this, tempUiObj->iTaskView->configData.obexTimeout);
				
				// Start OBEX Put
				logWithLookup(_L("OBEX Put"), CLogger::logOBEX);
				iStatus = KRequestPending;
				obexClient->Put(*obexFile, iStatus);
				SetActive();
			}
			else
			{
				TBuf<30> errText;
				
				errText.Format(_L("OBEX connect failure %d"), iStatus);
				logWithLookup(errText, CLogger::logOBEX);
				startRestTime.HomeTime();
				currentNoDevicePause = STACK_COOLOFF_COUNTDOWN;
				firstRest = ETrue;
				
				// Mark device as cached and timestamp it
				btDevice[currentDeviceIndex].isBluejacked = 3;
				btDevice[currentDeviceIndex].lastBluejackedTime.HomeTime();				
				rest();
			}
		}
		else if (scannerStatus == isOBEXSearch)
		{
			// I could have passed this via iStatus too.
			if (obexSearcher.RFCOMMPort > -1)
			{
				logWithLookup(_L("OBEX found"), CLogger::logOBEX);
				btDevice[currentDeviceIndex].obexPort = obexSearcher.RFCOMMPort;
				obexBTProtoInfo.iTransport.Copy(_L("RFCOMM"));
				obexBTProtoInfo.iAddr.SetBTAddr(btDevice[currentDeviceIndex].btDevAddr);
				obexBTProtoInfo.iAddr.SetPort(obexSearcher.RFCOMMPort);
				if (obexClient)
				{
					delete obexClient;
					obexClient = NULL;
				}
			    obexClient = CObexClient::NewL(obexBTProtoInfo);
				scannerStatus = isOBEXConnect;
			    iStatus = KRequestPending;
			    
   				// Start the timeout counter
				timeoutCounter.startTimer(this, tempUiObj->iTaskView->configData.obexTimeout);
			    
			    // Start OBEX connect
	    	    obexClient->Connect(iStatus);
	    	    SetActive();
			}
			else
			{
				TBuf<30> errText;
				
				errText.Format(_L("OBEX search error %d"), iStatus);
				logWithLookup(errText, CLogger::logOBEX);
				if (obexRetries > 0)
				{
					obexRetries--;
					iStatus = KRequestPending;
					scannerStatus = isOBEXSearch;
					obexSearcher.start(&iStatus, &(btDevice[currentDeviceIndex].btDevAddr));
					SetActive();
				}
				else
				{
					startRestTime.HomeTime();
					currentNoDevicePause = STACK_COOLOFF_COUNTDOWN;
					firstRest = ETrue;
				
					// Remove device from list
					btDevice[currentDeviceIndex].isBluejacked = 0;
					rest();
				}
			}			
		}
		else if (scannerStatus == isShutdown)
		{
			stopScan();
		}
		else if (scannerStatus == isInquiring)
		{	
			// Found a device?
			if (iStatus == KErrNone)
			{
				TBTSockAddr iBTSockAddr(nameEntry().iAddr);
				insertAndUpdateBTList(iBTSockAddr.BTAddr());
				/*
				btDevAddr[numBTFound] = iBTSockAddr.BTAddr();
				// Sometimes, corrupted memory comes back. Don't know why. We assume
				// the first byte must be zero
				if (btDevAddr[numBTFound][0] == 0)
				{
					if (resolveAction != KHostResInquiry)
						btNames[numBTFound] = nameEntry().iName;
					logWithLookup(_L("Found device"));			
					numBTFound++;
					currentNoDeviceAttempts = 0;
				}
				*/
				iStatus = KRequestPending;
				hostResolver.Next(nameEntry, iStatus);
				SetActive();
			}
			else
			{
				if (hostResolverOpen)
				{
					hostResolver.Close();
					hostResolverOpen = EFalse;
				}
				_LIT(txt, "No more devices found");
				logHelper->updateLog(&txt, 0, CLogger::logScanning);
				currentDeviceIndex = -1;
				bluejackNextDevice();
			}
		}
	}
}

TBool CScanner::getNextBTDeviceIndex(void)
{
	for (int i = (currentDeviceIndex + 1); i < MAX_BTDEV_ADDR; i++)
	{
		if (btDevice[i].isBluejacked == 1)
		{
			currentDeviceIndex = i;
			return ETrue;
		}		
	}
	currentDeviceIndex = -1;
	return EFalse;
}

// Returns TRUE if this device was inserted into the list (i.e. new device) else FALSE
TBool CScanner::insertAndUpdateBTList(TBTDevAddr btDevAddr)
{
	TInt cacheIndexToOverwrite = -1;
	TBool isExist = EFalse;
	TBool isInserted = EFalse;
	TTime currentTime;
	TTimeIntervalSeconds interval, oldInterval;
	
	// Need to be fair to all devices. So take a snapshot of time here rather than in each
	// loop iteration
	currentTime.HomeTime();
	
	// First, make sure this device does not exist in the list.
	for (int i = 0; i < MAX_BTDEV_ADDR; i++)
	{
		// Check if we found the device before. This means the device entry is either:
		// - a non-bluejacked device
		// - bluejacked but waiting for decay; OR
		// - a cached entry
		// In either case, it means we shouldn't insert anything into the array
		if (btDevice[i].btDevAddr == btDevAddr)
		{
			// Is this a device that was found before?
			if (btDevice[i].isBluejacked == 1)
			{
				isExist = ETrue;
				currentDeviceIndex = i;
				logWithLookup(_L("Found device in cache."), CLogger::logScanning);
				break;
				// We can leave the obex port in its unknown state. getNextBluejackDevice()
				// will take care of it
			}
			// Check if it was bluejacked but waiting for decay
			else if (btDevice[i].isBluejacked == 2)
			{
				if (currentTime.SecondsFrom(btDevice[i].lastBluejackedTime, interval) != 0)
					interval = -1;
				CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
				// Decay still not expired?
				if (interval.Int() < tempUiObj->iTaskView->configData.decayTime)
				{
					isExist = ETrue;
					
					// Generate a log entry
					currentDeviceIndex = i;
					logWithLookup(_L("Found device. Waiting for decay."), CLogger::logScanning);
					
					break;
				}
				// This means its decay time is up!
				else
				{
					isExist = ETrue;
					currentDeviceIndex = i;
					logWithLookup(_L("Found device in cache."), CLogger::logScanning);
					btDevice[i].isBluejacked = 1;
					break;
				}
			}
			// A cached entry?
			else if (btDevice[i].isBluejacked == 3)
			{
				isExist = ETrue;
				
				// Generate a log entry
				currentDeviceIndex = i;
				logWithLookup(_L("Found device in cache."), CLogger::logScanning);
				
				// Reset the item's status to be a newly found item
				btDevice[i].isBluejacked = 1;
				
				break;
			}
		}
	}
	
	// Ok, device wasn't found. We should insert it. It may be possible our array is
	// full (with cache and decaying entries). In that case, we should issue a message
	// to the user informing thus
	if (!isExist)
	{
		// Only the following types of array entries can be overwritten.
		// - empty entries (isBluejacked == 0)
		// - decaying entries who's decay time is up
		// - cached entries

		oldInterval = -1;
		CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());		
		for (int i = 0; i < MAX_BTDEV_ADDR; i++)
		{
			if (currentTime.SecondsFrom(btDevice[i].lastBluejackedTime, interval) != 0)
				interval = -1;

			// Is this a decaying item? If so, check if its time is up
			if (btDevice[i].isBluejacked == 2)
			{
				if (interval.Int() >= tempUiObj->iTaskView->configData.decayTime)
				{
					// This entry's decay time is up!
					isInserted = ETrue;
					btDevice[i].btDevAddr = btDevAddr;
					if (resolveAction != KHostResInquiry)
						btDevice[i].btNames = nameEntry().iName;
					btDevice[i].isBluejacked = 1;
					// Since this is a new entry, we haven't done a SDP. Set port to -1
					btDevice[i].obexPort = -1;
					currentDeviceIndex = i;
					break;
				}
			}
			// Is this an empty entry? If so insert immediately!
			else if (btDevice[i].isBluejacked == 0)
			{
				isInserted = ETrue;
				btDevice[i].btDevAddr = btDevAddr;
				if (resolveAction != KHostResInquiry)
					btDevice[i].btNames = nameEntry().iName;
				btDevice[i].isBluejacked = 1;				
				// Since this is a new entry, we haven't done a SDP. Set port to -1
				btDevice[i].obexPort = -1;
				
				currentDeviceIndex = i;
				break;
			}
			// Is this a cached entry? If so take the one with the oldest timestamp
			else if (btDevice[i].isBluejacked == 3)
			{
				if (currentTime.SecondsFrom(btDevice[i].lastBluejackedTime, interval) != 0)
					interval = -1;
				if (interval > oldInterval)
					cacheIndexToOverwrite = i;
			}
		}
		
		// Did we insert?
		if (isInserted)
			logWithLookup(_L("Found device"), CLogger::logScanning);
		// Did we find a suitable cached entry to overwrite?
		else if (cacheIndexToOverwrite > -1)
		{
			logWithLookup(_L("Found device (cache full)"), CLogger::logScanning);
			btDevice[cacheIndexToOverwrite].btDevAddr = btDevAddr;
			if (resolveAction != KHostResInquiry)
				btDevice[cacheIndexToOverwrite].btNames = nameEntry().iName;
			btDevice[cacheIndexToOverwrite].isBluejacked = 1;
			// Since this is a new entry, we haven't done a SDP. Set port to -1
			btDevice[cacheIndexToOverwrite].obexPort = -1;
		}
		// None of the above. This means buffer is full :(
		else
		{
			TBuf<35> theText;
			
			theText.Copy(_L("Device buffer full. Skipping."));
			if (resolveAction == KHostResInquiry)
			{
				TBuf<65> temp;	
				temp.Format(_L("%x:%x:%x:%x:%x:%x - "), btDevAddr[0], btDevAddr[1], 
				btDevAddr[2], btDevAddr[3], btDevAddr[4], btDevAddr[5]);
				temp.Append(theText);
				logHelper->updateLog(&temp, 0, CLogger::logScanning);
			}
			else
			{
				TBuf<sizeof(THostName) + 35> temp;
				temp.Copy(nameEntry().iName);
				temp.Append(_L(" - "));
				temp.Append(theText);
				logHelper->updateLog(&temp, 0, CLogger::logScanning);
			}
		}
	}	
	currentNoDeviceAttempts = 0;
	return isInserted;
}

void CScanner::DoCancel()
{
	if (obexClient)
	{
		//obexClient->Abort();
		delete obexClient;
		obexClient = NULL;
	}
	if (obexFile)
	{
		delete obexFile;
		obexFile = NULL;
	}

	if (hostResolverOpen)
	{
		hostResolver.Close();
		hostResolverOpen = EFalse;
	}
}

/*************************************************************
*
* Search for OBEX and find its RFCOMM port
*
**************************************************************/

CFindOBEX::CFindOBEX()
{
	iSdpSearchPattern = NULL;
	iMatchList = NULL;
	iAgent = NULL;
}

void CFindOBEX::StartListL(CSdpAttrValueList& /*aList*/)
{
}

void CFindOBEX::EndListL()
{
}

void CFindOBEX::releaseObjects(void)
{
	if (iAgent)
	{
		delete iAgent;
		iAgent = NULL;
	}
	if (iSdpSearchPattern)
	{
		delete iSdpSearchPattern;
		iSdpSearchPattern = NULL;
	}
	if (iMatchList)
	{
		delete iMatchList;
		iMatchList = NULL;
	}
}

void CFindOBEX::start(TRequestStatus* theStatus, const TBTDevAddr* theAddress)
{
	releaseObjects();
	localStatus = theStatus;
	
    iSdpSearchPattern = CSdpSearchPattern::NewL();
    iSdpSearchPattern->AddL(KL2CAP);
    iMatchList = CSdpAttrIdMatchList::NewL();
    iMatchList->AddL(TAttrRange(0x0000, 0x0005));
    iAgent = CSdpAgent::NewL(*this, *theAddress);
	iAgent->SetRecordFilterL(*iSdpSearchPattern);
	RFCOMMPort = -1;

	// This is an asynchronous call. On completion it calls
	// MSdpAgentNotifier::NextRecordRequestComplete() 
    iAgent->NextRecordRequestL();
}

void CFindOBEX::searchDone(TUint errorNumber)
{
	releaseObjects();
	User::RequestComplete(localStatus, errorNumber);
}

// Called when an service record request (CSdpAgent::NextRecordRequestL()) 
// operation completes.
void CFindOBEX::NextRecordRequestComplete(TInt aError, TSdpServRecordHandle aHandle, TInt aTotalRecordsCount)
{
	if (foundOBEX == 8)
	{
		searchDone(0);
		return;
	}
	foundOBEX = 0;
	if (aError != KErrNone)
	{
		searchDone(aError);
		return;
	}
	if (aTotalRecordsCount > 0)
	{
		/*
		Gets the specified attribute for a remote service. The function is asynchronous: 
		on completion, it calls MSdpAgentNotifier::AttributeRequestComplete(). 
		If the attribute is found, the function passes it by also calling 
		MSdpAgentNotifier::AttributeRequestResult().
		*/
        iAgent->AttributeRequestL(aHandle, *iMatchList);
	}
	else
	{
		searchDone(KErrEof);
		return;
	}
}

// Called when an attribute request (CSdpAgent::AttributeRequestL()) finds 
// that there are no more attributes to be returned.
void CFindOBEX::AttributeRequestComplete(TSdpServRecordHandle /*aHandle*/, TInt aError)
{
	if (aError == KErrNone)
	{
		if (foundOBEX != 8)
			iAgent->NextRecordRequestL();
		else
		{
			searchDone(aError);
			return;
		}
	}
	else
	{
		searchDone(aError);
		return;
	}
}

// Called by the attribute request function (CSdpAgent::AttributeRequestL()) to pass the 
// results of a successful attribute request.
void CFindOBEX::AttributeRequestResult(TSdpServRecordHandle /*aHandle*/, TSdpAttributeID aAttrID, CSdpAttrValue* aAttrValue)
{

	// We're past the protocol descriptor list
	if (aAttrID > 0x4)
		foundOBEX = 0;
		
	if (aAttrID == 0x4)
	{
		// Did we previously discover a service class attribute?
		if (foundOBEX == 3)
		{
			foundOBEX = 4;
		}
	}
	else if (aAttrID == 0x1)
	{
		foundOBEX = 1;
	}
	
	// When called on attributes that are not lists (not DEA or DES), then this calls 
	// MSdpAttributeValueVisitor::VisitAttributeValueL(), passing the attribute value
	// object itself (i.e. *this), and the value attribute type.
	if ((foundOBEX != 8) && (aAttrID <= 0x4))
	{
		aAttrValue->AcceptVisitorL(*this);
    
	}
    // Ownership has been transferred
    delete aAttrValue;
}

void CFindOBEX::VisitAttributeValueL(CSdpAttrValue& aValue, TSdpElementType aType)
{
	if (foundOBEX == 8)
		return;
		
	// Looking for service class attribute
	if (foundOBEX == 1)
	{
		if (aType == ETypeDES)
			foundOBEX = 2;
	}
	// Found service class attribute. Is it OBEX?
	else if (foundOBEX == 2)
	{
		if (aType == ETypeUUID)
		{
			if (aValue.UUID() == 0x1105)
				foundOBEX = 3;
		}
	}
	// Found OBEX service class and Protocol descriptor list. Is this RFCOMM?
	else if (foundOBEX == 4)
	{
		if (aType == ETypeDES)
			foundOBEX = 5;
	}
	else if (foundOBEX == 5)
	{
		if (aType == ETypeDES)
			foundOBEX = 6;
	}
	else if (foundOBEX == 6)
	{
		if (aType == ETypeUUID)
		{
			// Is it RFCOMM?
			if (aValue.UUID() == 0x0003)
				foundOBEX = 7;
			else
				foundOBEX = 5;
		}
		else if (foundOBEX != 8)
			foundOBEX = 5;
	}
	else if (foundOBEX == 7)
	{
		if (aType == ETypeUint)
		{
			RFCOMMPort = aValue.Uint();
			foundOBEX = 8;
		}
	}
}

/*************************************************************
*
* Timeout handler
*
**************************************************************/

CTimeout::CTimeout() : CTimer(EPriorityNormal)
{
	CTimer::ConstructL();
	CActiveScheduler::Add(this);
}

void CTimeout::RunL(void)
{
	TTime currentTime;
	TTimeIntervalSeconds interval;
			
	currentTime.HomeTime();
	if (currentTime.SecondsFrom(timeoutStart, interval) != 0)
		interval = localObexTimeout;
	if (interval.Int() >= localObexTimeout)
	{
		// Timed out. Is the scanner object still waiting?
		if (scanObj->IsActive())
		{
			_LIT(txt, "Timeout...");
			// This is hard coded to be an OBEX log category since timeout is only used
			// by OBEX calls
			scanObj->logHelper->updateLog(&txt, 0, CLogger::logOBEX);
			scanObj->timeoutAbort();
		}
	}
	else
		After(TIMER_GRANULARITY);
}

void CTimeout::startTimer(CScanner *theScanObj, TInt obexTimeout)
{
	scanObj = theScanObj;
	localObexTimeout = obexTimeout;
	timeoutStart.HomeTime();
	stopTimer();
	After(TIMER_GRANULARITY);
}

void CTimeout::stopTimer(void)
{
	if (IsActive())
		Cancel();
}

/*************************************************************
*
* Bluebeamer. Quite similar to scanner class
*
**************************************************************/

CBlueBeam::CBlueBeam() : CTimer(EPriorityNormal)
{
	beamerStatus = isIdle;
	selectedFiles = 0;
	logHelper = new (ELeave) CLogger;
	logHelper->controlId = cBluebeamLog;
	CTimer::ConstructL();
	CActiveScheduler::Add(this);
}

CBlueBeam::~CBlueBeam()
{
	delete logHelper;
}

void CBlueBeam::startBeam(TBool sendOrJack, TBool *autoSave, TBool *showSysMsg)
{
	if (sendOrJack)
	{
		CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
		if (selArray)
		{
			delete selArray;
			selArray = NULL;
		}
		// Make a copy of the selection indexes so we can track the bluebeam progress
		selArray = new (ELeave) CArrayFixFlat<TInt>(tempUiObj->iFileView->cFileListBox->SelectionIndexes()->Count());
		tempUiObj->iFileView->cFileListBox->View()->GetSelectionIndexesL(selArray);
	}
	localShowSysMsg = showSysMsg;
	logHelper->localAutoSave = autoSave;
	localSendOrJack = sendOrJack;
	beamerStatus = isStarting;
	if (selectDevice() == KErrNone)
		doBeam();
	else
		beamerStatus = isIdle;
}

TInt CBlueBeam::selectDevice(void)
{
	CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());

	tempUiObj->isAppUiBusy = ETrue;
	CQBTUISelectDialog* selectDialog = new (ELeave) CQBTUISelectDialog(devAddress, devName, devClass, CQBTUISelectDialog::EQBTDeviceFilterAll, EMajorServiceObjectTransfer, EFalse);
	if (selectDialog->LaunchSingleSelectDialogLD())
	{
		tempUiObj->isAppUiBusy = EFalse;
		return KErrNone;
	}
	else
	{
		tempUiObj->isAppUiBusy = EFalse;
		return KErrCancel;
	}
}

void CBlueBeam::doBeam(void)
{
	// Find OBEX
	iStatus = KRequestPending;
	beamerStatus = isOBEXSearch;
	obexRetries = OBEX_RETRIES;
	currentDeviceIndex = 0;
	obexSearcher.start(&iStatus, &devAddress);
	if (localSendOrJack)
	{
		CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
	
		tempUiObj->isAppUiBusy = ETrue;
		bluebeamLogDialog = new (ELeave) CBluebeamLogDialog;
		bluebeamLogDialog->ExecuteLD(R_DIALOG_BLUEBEAMLOG);
		bluebeamLogDialog->ButtonGroupContainer().DimCommand(EEikBidOk, ETrue);
		logHelper->logDialog = bluebeamLogDialog;
		selectedFiles = tempUiObj->iFileView->cFileListBox->SelectionIndexes()->Count();
	}
	logHelper->logBuffer = logBuffer;
	_LIT(txt, "OBEX search");
	logWithName(&txt);
	SetActive();
}

void CBlueBeam::sendOBEXFile(void)
{
	TPtrC tempText, selectedFile;
	
	if (obexFile)
	{
		delete obexFile;
		obexFile = NULL;
	}
    obexFile = CObexFileObject::NewL(TPtrC(NULL, 0));

	if (localSendOrJack)
	{
		CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
		tempText.Set(tempUiObj->iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint(tempUiObj->iFileView->cFileListBox->SelectionIndexes()->At(selectedFiles)));
		TextUtils::ColumnText(selectedFile, 0, &tempText);
		currentFileName.Copy(tempUiObj->iFileView->currentPath);
		currentFileName.Append(selectedFile);
	}
	
	TInt err;
	TRAP(err, obexFile->InitFromFileL(currentFileName));
	if (err)
	{	
		User::InfoPrint(_L("Couldn't open file"));
		TBuf<sizeof(TFileName) + sizeof(TPath) + 20> txt;
		txt.Copy(_L("Couldn't open file "));
		txt.Append(currentFileName);
		logHelper->updateLog(&txt, 0, CLogger::logAll);
		iStatus = KErrNone;
		beamerStatus = isOBEXNextFile;
		delete obexFile;
		obexFile = NULL;
		// On with the next file
		After(1);
	}
	else
	{	
		TBuf<sizeof(TFileName) + sizeof(TPath) + 10> txt;
		txt.Copy(_L("Sending "));
		txt.Append(currentFileName);
		logWithName(&txt);
		iStatus = KRequestPending;
		beamerStatus = isOBEXSend;
		obexClient->Put(*obexFile, iStatus);
		SetActive();
	}
}

void CBlueBeam::logWithName(const TDesC *buffer)
{
	TBuf<sizeof(TFileName) + sizeof(TPath) + sizeof(THostName) + 30> txt;
	txt.Copy(devName);
	txt.Append(_L(" - "));
	txt.Append(*buffer);
	logHelper->updateLog(&txt, 0, CLogger::logAll);
}

void CBlueBeam::RunL()
{
	if (beamerStatus == isOBEXSearch)
	{
		// I could have passed this via iStatus too.
		if (obexSearcher.RFCOMMPort > -1)
		{
			_LIT(txt, "Connecting...");
			logWithName(&txt);
			obexBTProtoInfo.iTransport.Copy(_L("RFCOMM"));
			obexBTProtoInfo.iAddr.SetBTAddr(devAddress);
			obexBTProtoInfo.iAddr.SetPort(obexSearcher.RFCOMMPort);
			if (obexClient)
			{
				delete obexClient;
				obexClient = NULL;
			}
		    obexClient = CObexClient::NewL(obexBTProtoInfo);
			beamerStatus = isOBEXConnect;
		    iStatus = KRequestPending;

		    // Start OBEX connect
    	    obexClient->Connect(iStatus);
    	    SetActive();
		}
		else
		{
			if (obexRetries > 0)
			{
				_LIT(txt, "OBEX search error. Retrying.");
				logWithName(&txt);
				obexRetries--;
				iStatus = KRequestPending;
				beamerStatus = isOBEXSearch;
				obexSearcher.start(&iStatus, &devAddress);
				SetActive();
			}
			else
			{
				_LIT(txt, "OBEX lookup failed. Aborting.");
				logWithName(&txt);
				beamerStatus = isIdle;
			}
		}			
	}
	else if ((beamerStatus == isOBEXConnect) || (beamerStatus == isOBEXNextFile))
	{
		if (iStatus == KErrNone)
		{
			selectedFiles--;
			if (selectedFiles >= 0)
				sendOBEXFile();
			else
			{
				_LIT(txt, "Finished. Disconnecting...");
				logWithName(&txt);
				iStatus = KRequestPending;
				beamerStatus = isOBEXDisconnect;
				obexClient->Disconnect(iStatus);
				SetActive();
			}
		}
		else
		{
			_LIT(txt, "OBEX connect failure.");
			logWithName(&txt);
			beamerStatus = isIdle;
		}
	}
	else if (beamerStatus == isOBEXSend)
	{
		if (iStatus == KErrNone)
		{
			if (*localShowSysMsg)
			{
				if (localSendOrJack == EFalse)
					User::InfoPrint(_L("Bluejacked!"));
			}
			
			if (localSendOrJack)
			{
				// Unmark the file in the filebrowser
				CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
				tempUiObj->iFileView->cFileListBox->View()->VerticalMoveToItemL(selArray->At(selectedFiles), CListBoxView::EDisjointSelection);
			}

			selectedFiles--;
			if (selectedFiles >= 0)
			{
				sendOBEXFile();
				return;
			}
			else
			{
				_LIT(txt, "Finished. Disconnecting...");
				logWithName(&txt);
				if (obexFile)
				{
					delete obexFile;
					obexFile = NULL;
				}
			}
		}
		else
		{
			_LIT(txt, "OBEX PUT failure. Disconnecting...");
			logWithName(&txt);
		}
		iStatus = KRequestPending;
		beamerStatus = isOBEXDisconnect;
		obexClient->Disconnect(iStatus);
		SetActive();
	}
	else if (beamerStatus == isOBEXDisconnect)
	{
		if (iStatus == KErrNone)
		{
			_LIT(txt, "Done.");
			logWithName(&txt);
		}
		else
		{
			_LIT(txt, "Error disconnecting. Terminated.");
			logWithName(&txt);
		}
		beamerStatus = isIdle;
	}
	if (beamerStatus == isIdle)
	{
		if (localSendOrJack)
		{
			CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
			if (tempUiObj->isAppUiBusy)
				bluebeamLogDialog->ButtonGroupContainer().DimCommand(EEikBidOk, EFalse);
			if (selArray)
			{
				delete selArray;
				selArray = NULL;
			}
		}
		if (obexFile)
		{
			delete obexFile;
			obexFile = NULL;
		}
		if (obexClient)
		{
			delete obexClient;
			obexClient = NULL;
		}
	}
}

void CBlueBeam::DoCancel()
{
	if (IsActive())
	{
		if (beamerStatus != isOBEXNextFile)
		{
			if (obexClient)
			{
				delete obexClient;
				obexClient = NULL;
			}
			if (obexFile)
			{
				delete obexFile;
				obexFile = NULL;
			}
		}
		beamerStatus = isIdle;
		
		// This is kludgy but it works.
		TRequestStatus* tempStatus;		
		tempStatus = &iStatus;
		User::RequestComplete(tempStatus, KErrNone);
		User::WaitForRequest(iStatus);
	}
	else
		beamerStatus = isIdle;
}