#include "SMan.h"
#include <SMan.rsg>

/*************************************************************
*
* Application object
*
**************************************************************/

CApaDocument* CSMan2Application::CreateDocumentL()
{
	CSMan2Document*	doc = new (ELeave) CSMan2Document(*this);
	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();
	return doc;
}

/*************************************************************
*
* Document object
*
**************************************************************/

CSMan2Document::CSMan2Document(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CSMan2Document::~CSMan2Document()
{
}

void CSMan2Document::ConstructL()
{
}

CEikAppUi* CSMan2Document::CreateAppUiL()
{
	return new(ELeave) CSMan2AppUi;
}

/*************************************************************
*
* AppUi object
*
**************************************************************/

CSMan2AppUi::~CSMan2AppUi()
{
	RemoveFromStack(iActiveView);
	delete iTaskView;
	delete iFileView;
}

void CSMan2AppUi::HandleForegroundEventL(TBool aForeground)
{
	if (aForeground)
	{
		// Update the screen
		iTaskView->refreshTaskList(TODO_REFRESH);
		iTaskView->isFocused = ETrue;
		// Reactive marquee if we are in file view
		if (iMenuBar == R_BLUEBEAM_MENUBAR)
		{
			if (!(iFileView->currentPathScroller->IsActive()))
				iFileView->currentPathScroller->startMarquee(iFileView->labelCurrentFolder, &(iFileView->currentPath));
		}
	}
	else
	{
		// Stop the marquee to conserve battery power
		if (iFileView->currentPathScroller->IsActive())
			iFileView->currentPathScroller->Cancel();
		iTaskView->isFocused = EFalse;
	}
	CEikAppUi::HandleForegroundEventL(aForeground);
}

void CSMan2AppUi::switchView(TInt viewId)
{
	CCoeControl *iOldView;
	
	iOldView = iActiveView;
	RemoveFromStack(iActiveView);
	if (viewId == 0)
	{
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_DEFAULT_MENUBAR, EFalse);
		iMenuBar = R_DEFAULT_MENUBAR;
		iActiveView = iTaskView;
		// Save battery
		if (iFileView->currentPathScroller->IsActive())
			iFileView->currentPathScroller->Cancel();		
	}
	else if (viewId == 1)
	{
		if (!(iFileView->currentPathScroller->IsActive()))
			iFileView->currentPathScroller->startMarquee(iFileView->labelCurrentFolder, &(iFileView->currentPath));
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_BLUEBEAM_MENUBAR, EFalse);
		iMenuBar = R_BLUEBEAM_MENUBAR;
		iActiveView = iFileView;
	}
	currentViewType = viewId;
	iActiveView->MakeVisible(ETrue);
	iOldView->MakeVisible(EFalse);
	iActiveView->DrawNow();
	AddToStackL(iActiveView);
}

void CSMan2AppUi::ConstructL()
{
	isAppUiBusy = EFalse;
	BaseConstructL();
	iTaskView = new (ELeave) CSMan2TaskListView;
	iTaskView->ConstructL(ClientRect());
	iFileView = new (ELeave) CSMan2FileListView;
	iFileView->ConstructL(ClientRect());

	currentViewType = 0;
	iActiveView = iTaskView;	
	iMenuBar = R_DEFAULT_MENUBAR;
	iFileView->MakeVisible(EFalse);
	AddToStackL(iActiveView);
}

void CSMan2AppUi::HandleCommandL(TInt aCommand)
{
	/*************************************************************
	*
	* Commands for 2nd view - File View
	*
	**************************************************************/
	if (aCommand == cmdBluebeam_SelectAll)
	{
		TPtrC tempText, entryType;

		for (int i = 0; i < iFileView->cFileListBox->Model()->ItemTextArray()->MdcaCount(); i++)
		{
			tempText.Set(iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint(i));
			TextUtils::ColumnText(entryType, 1, &tempText);
			if (entryType.Compare(_L("<dir>")) != 0)
				iFileView->cFileListBox->View()->SelectItemL(i);
		}
	}
	else if (aCommand == cmdBluebeam_DeselectAll)
	{
		TPtrC tempText, entryType;

		for (int i = 0; i < iFileView->cFileListBox->Model()->ItemTextArray()->MdcaCount(); i++)
		{
			tempText.Set(iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint(i));
			TextUtils::ColumnText(entryType, 1, &tempText);
			if (entryType.Compare(_L("<dir>")) != 0)
				iFileView->cFileListBox->View()->DeselectItem(i);
		}
	}
	else if (aCommand == cmdBluebeam_Send)
	{
		switchView(0);
		if (iFileView->cFileListBox->SelectionIndexes()->Count() > 0)
		{
			// If the first parameter is ETrue, the file name list does not need to be
			// populated. Saves memory.
			/*
			TPtrC tempText, selectedFile;
			
			iFileView->dummy = EFalse;
			iFileView->blueBeam->selectedFiles = iFileView->cFileListBox->SelectionIndexes()->Count();
			for (int i = 0; i < iFileView->blueBeam->selectedFiles; i++)
			{
				tempText.Set(iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint(iFileView->cFileListBox->SelectionIndexes()->At(i)));
				TextUtils::ColumnText(selectedFile, 0, &tempText);
				iFileView->blueBeam->fileNames[i] = iFileView->currentPath;
				iFileView->blueBeam->fileNames[i].Append(selectedFile);
			}
			*/
			iFileView->blueBeam->logHelper->logDialog = NULL;
			iFileView->blueBeam->startBeam(ETrue, &(iFileView->dummy), &(iTaskView->configData.showBluejackSysMsg));
		}
		else
			User::InfoPrint(_L("No files selected"));
	}
	else if (aCommand == cmdBluebeam_Cancel)
	{
		switchView(0);
		isAppUiBusy = EFalse;
	}
	else if (aCommand == cmdBluebeam_Drive_A)
	{
		iFileView->fillListBox(_L("A:\\"));
	}
	else if (aCommand == cmdBluebeam_Drive_C)
	{
		iFileView->fillListBox(_L("C:\\"));
	}
	else if (aCommand == cmdBluebeam_Drive_D)
	{
		iFileView->fillListBox(_L("D:\\"));
	}
	else if (aCommand == cmdBluebeam_Drive_Z)
	{
		iFileView->fillListBox(_L("Z:\\"));
	}
	/*************************************************************
	*
	* Commands for default view - Task View
	*
	**************************************************************/
	else if (aCommand == cmdLogFlags)
	{
		CLogLevelDialog* logLevelDialog = new (ELeave) CLogLevelDialog(&(iTaskView->configData.logFlags));
		isAppUiBusy = ETrue;
		if (logLevelDialog->ExecuteLD(R_DIALOG_LOGLEVELS) == EEikBidOk)
			iTaskView->saveConfig();
		isAppUiBusy = EFalse;
	}
	else if (aCommand == cmdShowHiddenTask)
	{
		iTaskView->configData.showHiddenTasks = !iTaskView->configData.showHiddenTasks;
		iTaskView->saveConfig();
		iTaskView->refreshTaskList(TODO_REFRESH);
	}
	else if (aCommand == cmdNoDevicePause)
	{
		CEikDialog* noDevicePauseDialog = new(ELeave) CBluejackTimerDialog(&(iTaskView->configData.noDevicePause), &(iTaskView->configData.noDeviceAttempts), &(iTaskView->configData.decayTime), &(iTaskView->configData.obexTimeout));
		isAppUiBusy = ETrue;
		if (noDevicePauseDialog->ExecuteLD(R_DIALOG_NODEVICEPAUSE) == EEikBidOk)
			iTaskView->saveConfig();
		isAppUiBusy = EFalse;
	}
	else if (aCommand == cmdForceEndTask)
	{
		iTaskView->configData.forceEndTask = !iTaskView->configData.forceEndTask;
		iTaskView->saveConfig();
	}
	else if (aCommand == cmdFlipFlush)
	{
		if (!iTaskView->configData.flipFlush)
		{
			if (iTaskView->snapshotTasks->Count() <= 0)
				iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_NOSNAPSHOT);
			else
				iTaskView->configData.flipFlush = !iTaskView->configData.flipFlush;
		}
		else
			iTaskView->configData.flipFlush = !iTaskView->configData.flipFlush;
		iTaskView->saveConfig();
	}
	else if (aCommand == cmdEndTask)
	{
		if (iTaskView->cTaskListBox->SelectionIndexes()->Count() > 0)
		{
			iTaskView->refreshTaskList(TODO_TERMINATE);			
		}
		else
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_NOMARKEDTASKS);
	}
	else if (aCommand == cmdBluejackAutoSave)
	{
		iTaskView->configData.autoSave = !iTaskView->configData.autoSave;
		iTaskView->saveConfig();
	}
	else if (aCommand == cmdFlushOut)
	{
		flushTasks();
	}
	else if (aCommand == cmdSnapshot)
	{
		CEikDialog* snapshotDialog = new(ELeave) CEikDialog;
		isAppUiBusy = ETrue;
		if (snapshotDialog->ExecuteLD(R_DIALOG_SNAPSHOTCONFIRM) == EEikBidYes)
			updateSnapShot();
		isAppUiBusy = EFalse;
	}
	else if (aCommand == cmdFixApplist)
	{
		TApaAppInfo dummy;
		
		iTaskView->saveConfig();
		FindApp(iTaskView->qFilemanUID, &dummy);
		iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_APPLISTFIXED);
	}
	else if (aCommand == cmdHotkey)
	{
		CEikDialog* dialog = new(ELeave) CHotkeyDialog(&(iTaskView->configData.iHotkeyDialogResult));
		isAppUiBusy = ETrue;
		if (dialog->ExecuteLD(R_DIALOG_HOTKEY) == EEikBidOk)
		{
			iTaskView->ToggleHotKey();
			if (iTaskView->saveConfig() != 0)
				iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVECONFIG);
		}
		isAppUiBusy = EFalse;
	}
	else if ((aCommand == cmdQuit) || (aCommand == EEikCmdExit))
	{ 
		if (iTaskView->oScanner->IsActive())
		{
			if (iTaskView->oScanner->scannerStatus == CScanner::isShutdown)
				iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_SCANNERSTOPPING);
			else
				iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_STOPSCANNER);
		}
		else if (iFileView->blueBeam->IsActive())
		{
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_STOPBEAM);
		}
		else if (iTaskView->bluejackOne->IsActive())
		{
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_STOPJACKONE);
		}
		else
			Exit();
	}
	else if (aCommand == cmdCompressHeap)
	{
		User::CompressAllHeaps();
		User::Heap().Compress();
		iTaskView->refreshTaskList(TODO_REFRESH);
	}
	else if (aCommand == cmdRefresh)
	{
		iTaskView->refreshTaskList(TODO_REFRESH);
	}
	else if ((aCommand == cmdBluesend) || (aCommand == cmdBluejackOne))
	{
		if (iTaskView->oScanner->scanStarted == EFalse)
		{
			if (aCommand == cmdBluesend)
			{
				// Initialize the bluesend log dialog
				if (currentViewType == 0)
				{
					isAppUiBusy = ETrue;
					switchView(1);
				}
			}
			else
			{
				TFileName vCard;
				
				iTaskView->bluejackOne->selectedFiles = 1;
				vCard.Copy(CEikonEnv::Static()->EikAppUi()->Application()->AppFullName());
				vCard.SetLength(vCard.Length() - 8);  // SMAN.APP = 8 chars
				vCard.Append(_L("SMAN.VCF"));
				iTaskView->bluejackOne->currentFileName = vCard;
				
				// bluejack one
				iTaskView->bluejackOne->logHelper->logDialog = NULL;				
				iTaskView->bluejackOne->startBeam(EFalse, &(iTaskView->configData.autoSave), &(iTaskView->configData.showBluejackSysMsg));
			}
		}
		else
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_STOPSCANNER);
	}
	else if (aCommand == cmdResolveName)
	{
		iTaskView->configData.resolveName = !iTaskView->configData.resolveName;
		if (iTaskView->saveConfig() != 0)
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVECONFIG);	
	}
	else if (aCommand == cmdNoHotkey)
	{
		iTaskView->configData.hotkeyEnabled = !iTaskView->configData.hotkeyEnabled;
		if (iTaskView->saveConfig() != 0)	
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVECONFIG);
		iTaskView->ToggleHotKey();
	}
	else if (aCommand == cmdBluejackSysMsg)
	{
		iTaskView->configData.showBluejackSysMsg = !iTaskView->configData.showBluejackSysMsg;
		if (iTaskView->saveConfig() != 0)
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVECONFIG);
	}
	else if (aCommand == cmdBluejackOnce)
	{
		iTaskView->configData.proxSweepOnce = !iTaskView->configData.proxSweepOnce;
		if (iTaskView->saveConfig() != 0)
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVECONFIG);
	}
	else if (aCommand == cmdAbout)
	{
		CEikDialog*	aboutDialog = new (ELeave) CEikDialog;
		isAppUiBusy = ETrue;
		aboutDialog->ExecuteLD(R_DIALOG_ABOUT);
		isAppUiBusy = EFalse;
/*
RWsSession oWindowSession;
CApaWindowGroupName* oWindowGroupName;
CArrayFixFlat<TInt>* windowGroupList;
TPtrC currentCaptionText;

// Simulate user pointer clicks
//TUid smal;
//smal.iUid = 0x10000083;
//LaunchApp(smal, _L(""));
oWindowSession.Connect();
windowGroupList = new CArrayFixFlat<TInt>(5);
oWindowGroupName = CApaWindowGroupName::NewL(oWindowSession);
oWindowSession.WindowGroupList(windowGroupList);

// Somehow FindWindowGroupIdentifier didn't work for me. :(
for (int i = 0; i < windowGroupList->Count(); i++)
{
 	oWindowGroupName->ConstructFromWgIdL(windowGroupList->At(i));
 	currentCaptionText.Set(oWindowGroupName->Caption());
 	if ((!oWindowGroupName->IsSystem()) && (!oWindowGroupName->Hidden()))
 	{
	 	if (currentCaptionText.Compare(_L("SMAN")) == 0)
	 	{
	 		TWsEvent theEvent;
	 		theEvent.SetType(EEventPointer);
	 		theEvent.SetHandle(oWindowSession.GetWindowGroupHandle(windowGroupList->At(i)));
	 		TPointerEvent& pointerEvent = *theEvent.Pointer();
	 		pointerEvent.iType = TPointerEvent::EButton1Down;
	 		pointerEvent.iPosition = TPoint(22, 150);
	 		pointerEvent.iParentPosition = TPoint(22 + 25, 150);
	  		if (oWindowSession.SendEventToWindowGroup(windowGroupList->At(i), theEvent) == KErrNone)
		  		User::InfoPrint(_L("Sent"));
		  	else
		  		User::InfoPrint(_L("Failure"));
	 	}
	 }
}
delete oWindowGroupName;
delete windowGroupList;
oWindowSession.Close();
*/
	}
	else if (aCommand == cmdQFileMan)
	{
		if (iTaskView->qFilemanUID.iUid != 0)
		{
			LaunchApp(iTaskView->qFilemanUID, _L(""));
		}
	}
	else if (aCommand == cmdBluejack)
	{
		if (iTaskView->oScanner->scanStarted == EFalse)
		{
			iTaskView->oScanner->logHelper->logDialog = NULL;
			iTaskView->oScanner->startScan(&(iTaskView->configData.autoSave), iTaskView->configData.resolveName, &(iTaskView->configData.showBluejackSysMsg), &(iTaskView->configData.proxSweepOnce), &(iTaskView->configData.noDeviceAttempts), &(iTaskView->configData.noDevicePause));
		}
		else
			iTaskView->oScanner->stopScan();
	}
	else if (aCommand == cmdBluejackLog)
	{
		CBluejackLogDialog *bluejackLogDialog = new (ELeave) CBluejackLogDialog(iTaskView->oScanner->logHelper);
		isAppUiBusy = ETrue;
		
		if (iTaskView->bluejackOne->beamerStatus != CBlueBeam::isIdle)
			iTaskView->bluejackOne->logHelper->logDialog = bluejackLogDialog;
		else if (iTaskView->oScanner->scannerStatus != CScanner::isIdle)
			iTaskView->oScanner->logHelper->logDialog = bluejackLogDialog;
			
		bluejackLogDialog->ExecuteLD(R_DIALOG_BLUEJACKLOG);
		
		if (iTaskView->bluejackOne->beamerStatus != CBlueBeam::isIdle)
			iTaskView->bluejackOne->logHelper->logDialog = NULL;
		else if (iTaskView->oScanner->scannerStatus != CScanner::isIdle)
			iTaskView->oScanner->logHelper->logDialog = NULL;
			
		isAppUiBusy = EFalse;
	}
	else if (aCommand == cmdBluejackMsg)
	{
		if (iTaskView->oScanner->scanStarted == EFalse)
		{
			CEikDialog* bluejackMsgDialog = new (ELeave) CBluejackMessageDialog(iTaskView->oScanner);
			isAppUiBusy = ETrue;
			bluejackMsgDialog->ExecuteLD(R_BLUEJACK_VCARD);
			isAppUiBusy = EFalse;
		}
		else
			iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_STOPSCANNER);
	}
	else if (aCommand == cmdControlPanel)
	{
		LaunchApp(iTaskView->cPanelUID, _L(""));
	}
	else if (aCommand == cmdShortcuts)
	{
		CArrayFixFlat<TUid> *tempUid;

		tempUid = new (ELeave) CArrayFixFlat<TUid>(NUM_SHORTCUTS);
		for (int i = 0; i < NUM_SHORTCUTS; i++)
		{
			tempUid->AppendL(iTaskView->configData.shortCutUid[i]);
		}
		CShortcutDialog* shortcutDialog = new (ELeave) CShortcutDialog(tempUid);
		isAppUiBusy = ETrue;
		if (shortcutDialog->ExecuteLD(R_DIALOG_SHORTCUT) == EEikBidOk)
		{
			for (int i = 0; i < NUM_SHORTCUTS; i++)
			{
				iTaskView->configData.shortCutUid[i] = tempUid->At(i);
			}
			if (iTaskView->saveConfig() != 0)
				iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVECONFIG);
		}
		isAppUiBusy = EFalse;
		delete tempUid;
	}
	else if ((aCommand >= cmdShort1) && (aCommand <= cmdShort7))
	{
		for (int i = 0; i < NUM_SHORTCUTS; i++)
		{
			if (aCommand == (cmdShort1 + i))
			{
				LaunchApp(iTaskView->configData.shortCutUid[i], _L(""));
				break;
			}
		}
	}
	else if (aCommand > cmdRunAppBase)
	{
		TUid tempUid;
		
		tempUid.iUid = aCommand - cmdRunAppBase;
		LaunchApp(tempUid, _L(""));
	}
}

void CSMan2AppUi::LaunchApp(TUid appUid, TFileName docName)
{
	RApaLsSession mySession;
	TInt runResult;
	TApaAppInfo aInfo;		
	
	// Don't launch if another copy is already running.

	runResult = KErrGeneral;
	TApaTaskList taskList(CEikonEnv::Static()->WsSession());
	TApaTask theTask = taskList.FindApp(appUid);
	if (theTask.Exists())
	{
		theTask.BringToForeground();
		runResult = KErrNone;
	}
	else
	{
		if (FindApp(appUid, &aInfo) == 0)
		{
			TRAPD(err, QikFileUtils::MakeAppDocumentFileNameL(docName, appUid));
			if (err == KErrNone)
			{
				TThreadId dummy;
				mySession.Connect();
				runResult = mySession.StartDocument(docName, appUid, dummy,	RApaLsSession::ESwitchFiles);
				mySession.Close();
			}
			else
			{
				CApaCommandLine* cmdln = CApaCommandLine::NewLC();
				cmdln->SetLibraryNameL(aInfo.iFullName);
				cmdln->SetCommandL(EApaCommandRun);
				mySession.Connect();
				runResult = mySession.StartApp(*cmdln);
				CleanupStack::PopAndDestroy(cmdln);
				mySession.Close();
			}
		
/*
			RProcess exeProc;
			_LIT(appRunPath, "z:\\System\\Programs\\AppRun.exe");
			_LIT(KGUIAppPath, "a:\\system\\apps\\viewer\\viewer.app");
			runResult = exeProc.Create(KAppRunPath, KGUIAppPath);
			if (runResult == KErrNone)
			{
			    exeProc.Resume();
			    exeProc.Close();
			}
*/
/*
			TBuf<sizeof(TPath) + sizeof(TFileName) + 5> fullDocName;
			CApaCommandLine *cmdLine = CApaCommandLine::NewLC();
			cmdLine->SetLibraryNameL(aInfo.iFullName);
			fullDocName.Copy(docName);
			if (docName == _L(""))
			{
				TBuf<sizeof(TFileName) + sizeof(TPath)> appName;
				
				appName.Copy(aInfo.iFullName);
				appName = appName.Right(appName.Length() - appName.LocateReverse(TChar(0x5c)) - 1);
				appName = appName.Left(appName.Length() - appName.LocateReverse(TChar(0x2e)));
				fullDocName.Copy(_L(""));
				fullDocName.Append(_L("C:\\DOCUMENTS\\"));
				fullDocName.Append(appName);
				fullDocName.Append(_L("\\"));
				fullDocName.Append(appName);
			}
			cmdLine->SetDocumentNameL(fullDocName);
			cmdLine->SetCommandL(EApaCommandRun);
			if (mySession.Connect() == KErrNone)
			{
				runResult = mySession.StartApp(*cmdLine);
				mySession.Close();
			}
			CleanupStack::PopAndDestroy();
*/
		}
	}
	if (runResult != KErrNone)
		iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRLAUNCHAPP);
}

void CSMan2AppUi::AppToForeground(void)
{
	RWsSession& oWindowSession = iEikonEnv->WsSession();
	RWindowGroup& oWindowGroup = iEikonEnv->RootWin();
	
	int windowID = oWindowGroup.Identifier();
	TApaTask oTApaTask(oWindowSession);
	oTApaTask.SetWgId(windowID);
	oTApaTask.BringToForeground();
}

void CSMan2AppUi::HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination)
{
	if ((aEvent.Type() == EEventKey) && (aEvent.Key()->iCode == sCaptureKeyCodes[iTaskView->configData.iHotkeyDialogResult]))
	{
		if (CEikonEnv::Static()->ScreenDevice()->CurrentScreenMode() != 0)
		{
			TBuf<64> buffer;
			
			CEikonEnv::Static()->ReadResource(buffer, R_TBUF_FLIPCLOSEDERR);
			User::InfoPrint(buffer);
		}
		else 
		{
			if (!iTaskView->isFocused)
				AppToForeground();
			else if (!isAppUiBusy)
			{
				//LaunchPopupMenuL(R_SHORTCUTS_MENU, TPoint(0, 42), EPopupTargetTopLeft, NULL);
				CEikonEnv::Static()->AppUiFactory()->MenuBar()->MoveHighlightToL(0, -1);
			}
		}
	} 
	else if (aEvent.Type() == EEventScreenDeviceChanged)
	{
		if (CEikonEnv::Static()->ScreenDevice()->CurrentScreenMode() == 0) // 0 = flip open
		{
			if (iTaskView->configData.iHotkeyDialogResult == 3)
				AppToForeground();
		}
		else
		{
			if (iTaskView->configData.flipFlush)
				flushTasks();
		}
	}
	CEikAppUi::HandleWsEventL(aEvent, aDestination);
}

void CSMan2AppUi::flushTasks(void)
{
	if (iTaskView->snapshotTasks->Count() <= 0)
	{
		iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_NOSNAPSHOT);
	}
	else
	{
		iTaskView->refreshTaskList(TODO_FLUSH);
		iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_TASKSFLUSHED);
	}
}

void CSMan2AppUi::updateSnapShot(void)
{
	iTaskView->snapshotTasks->Reset();
	iTaskView->refreshTaskList(TODO_SNAPSHOT);
	if (iTaskView->saveSnapShot() == 0)
		iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_SNAPSHOTUPDATED);
	else
		iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_ERRSAVESNAPSHOT);
}

TInt CSMan2AppUi::FindApp(TUid appUid, TApaAppInfo *aInfo)
{
	// Given an app UID, find if it is installed and get its app info

	RApaLsSession mySession;
	TApaAppInfo theAppInfo;
	TApaAppCapabilityBuf aCapabilityBuf;
	TInt errRes, iTimeout = 50;
	TTimeIntervalMicroSeconds32 iDelay = 100;
	
	// For some strange reason when I have just issued a command to write to the
	// disk (via saving snapshot or saving shortcuts), the app server always returns
	// an invalid application list (corrupted). I have to wait a while before the query
	// runs correct again....?????

	CEikonEnv::Static()->BusyMsgL(_L("Validating..."), TGulAlignment(EHRightVTop));
	mySession.Connect();
	mySession.GetAllApps();
	errRes = 2;
	while ((errRes != 0) & (iTimeout > 0))
	{
		while (mySession.GetNextApp(theAppInfo) == KErrNone)
		{
			errRes = 1;
			if (mySession.GetAppCapability(aCapabilityBuf, theAppInfo.iUid) == KErrNone)
			{
				// Only scan applications that are not hidden
				if (aCapabilityBuf().iAppIsHidden == EFalse)
				{
					if (theAppInfo.iUid == appUid)
					{
						aInfo->iUid = theAppInfo.iUid;
						aInfo->iFullName = theAppInfo.iFullName;
						aInfo->iCaption = theAppInfo.iCaption;
						errRes = 0;
						break;
					}
				}
			}
		}
		User::After(iDelay);
		iTimeout--;
	} 
	mySession.Close();
	CEikonEnv::Static()->BusyMsgCancel();
	return errRes;
}

void CSMan2AppUi::sizeToText(TInt64 size, TDes* desc)
{
	TInt64 origSize, decimals;
	TBuf<3> sizeName;
	
	origSize = size;
	if (size < 1024)
	{
		sizeName = _L("b)");
		decimals = 0;
	}
	else if (size < (1024 * 1024))
	{
		size = size / 1024;
		decimals = (origSize - (size * 1024)) / 1024;
		sizeName = _L("KB)");
	}
	else
	{
		size = size / (1024 * 1024);
		decimals = (origSize - (size * 1024 * 1024)) / 1024;
		sizeName = _L("MB)");
	}
	
	desc->Num(size);
	desc->Append(_L("."));
	desc->AppendFormat(_L("%2d"), decimals);
	if (size > 999)
		desc->Insert(desc->Length() - 6, _L(","));
	desc->Append(sizeName);
	desc->Insert(0, _L("]("));
}

void CSMan2AppUi::DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane)
{
	if (iMenuBar == R_BLUEBEAM_MENUBAR)
	{
		if (aMenuId == R_DRIVES_MENU)
		{
			// Get volume + free space information for all the drives.
			TVolumeInfo volInfo;
			TInt err;
			TBuf<KMaxFileName + 30> desc;
				
/*
			err = iEikonEnv->FsSession().Volume(volInfo, EDriveA);
			if (err == KErrNone)
				aMenuPane->SetItemTextL(cmdBluebeam_Drive_A, volInfo.iName);
*/
			err = iEikonEnv->FsSession().Volume(volInfo, EDriveC);
			if (err == KErrNone)
			{
				sizeToText(volInfo.iFree, &desc);
				desc.Insert(0, volInfo.iName);
				desc.Insert(0, _L("C:["));
				aMenuPane->SetItemTextL(cmdBluebeam_Drive_C, desc);
			}

			err = iEikonEnv->FsSession().Volume(volInfo, EDriveD);
			if (err == KErrNone)
			{
				sizeToText(volInfo.iFree, &desc);
				desc.Insert(0, volInfo.iName);
				desc.Insert(0, _L("D:["));
				aMenuPane->SetItemTextL(cmdBluebeam_Drive_D, desc);
			}
			else
				aMenuPane->SetItemDimmed(cmdBluebeam_Drive_D, ETrue);
/*
			err = iEikonEnv->FsSession().Volume(volInfo, EDriveZ);
			if (err == KErrNone)
				aMenuPane->SetItemTextL(cmdBluebeam_Drive_Z, volInfo.iName);
*/
		}
	}
	else if (iMenuBar == R_DEFAULT_MENUBAR)
	{
		if (aMenuId == R_EXTRAS_MENU)
		{
			if ((iTaskView->oScanner->IsActive()) || (iFileView->blueBeam->IsActive()) || (iTaskView->bluejackOne->IsActive()))
				aMenuPane->SetItemDimmed(cmdBluesend, ETrue);
			if (iTaskView->configData.hotkeyEnabled)
				aMenuPane->SetItemButtonState(cmdNoHotkey, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdNoHotkey, EEikMenuItemSymbolOn);
			if (!iTaskView->configData.flipFlush)
				aMenuPane->SetItemButtonState(cmdFlipFlush, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdFlipFlush, EEikMenuItemSymbolOn);
			if (!iTaskView->configData.forceEndTask)
				aMenuPane->SetItemButtonState(cmdForceEndTask, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdForceEndTask, EEikMenuItemSymbolOn);
			if (!iTaskView->configData.showHiddenTasks)
				aMenuPane->SetItemButtonState(cmdShowHiddenTask, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdShowHiddenTask, EEikMenuItemSymbolOn);
		}
		else if (aMenuId == R_RUNAPPLIST_MENU)
		{
			CEikMenuPaneItem::SData itemData;

			CShortcutDialog* shortcutDialog = new (ELeave) CShortcutDialog(NULL);
			shortcutDialog->populateAppArray();
			for (int i = 0; i < shortcutDialog->appListArray->MdcaCount(); i++)
			{
				itemData.iText.Copy(shortcutDialog->appListArray->MdcaPoint(i));
				itemData.iCommandId = cmdRunAppBase + shortcutDialog->appListUid->At(i).iUid;
				itemData.iFlags = 0;
				itemData.iCascadeId = 0;
				aMenuPane->AddMenuItemL(itemData);
			}
			delete shortcutDialog;
		}
		else if (aMenuId == R_OPTIONS_BLUEJACK_MENU)
		{
			if (iTaskView->configData.showBluejackSysMsg == EFalse)
				aMenuPane->SetItemButtonState(cmdBluejackSysMsg, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdBluejackSysMsg, EEikMenuItemSymbolOn);
			if (iTaskView->configData.resolveName == EFalse)
				aMenuPane->SetItemButtonState(cmdResolveName, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdResolveName, EEikMenuItemSymbolOn);
			if (iTaskView->configData.autoSave == EFalse)
				aMenuPane->SetItemButtonState(cmdBluejackAutoSave, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdBluejackAutoSave, EEikMenuItemSymbolOn);
			if (iTaskView->configData.proxSweepOnce == EFalse)
				aMenuPane->SetItemButtonState(cmdBluejackOnce, EEikMenuItemSymbolIndeterminate);
			else
				aMenuPane->SetItemButtonState(cmdBluejackOnce, EEikMenuItemSymbolOn);
		}
		else if (aMenuId == R_BLUEJACK_MENU)
		{
			if ((iTaskView->oScanner->IsActive()) || (iFileView->blueBeam->IsActive()) || (iTaskView->bluejackOne->IsActive()))
				aMenuPane->SetItemDimmed(cmdBluejackOne, ETrue);
			if ((iFileView->blueBeam->IsActive()) || (iTaskView->bluejackOne->IsActive()))
				aMenuPane->SetItemDimmed(cmdBluejack, ETrue);
			if (iTaskView->oScanner->scanStarted == EFalse)
				aMenuPane->SetItemButtonState(cmdBluejack, EEikMenuItemSymbolIndeterminate);
			else
			{
				if (iTaskView->oScanner->scannerStatus == CScanner::isShutdown)
					aMenuPane->SetItemDimmed(cmdBluejack, ETrue);
				else
					aMenuPane->SetItemButtonState(cmdBluejack, EEikMenuItemSymbolOn);
			}
		}
		else if (aMenuId == R_SHORTCUTS_MENU)
		{
			TApaAppInfo aInfo;
			
			if (FindApp(iTaskView->qFilemanUID, &aInfo) == 0)
			{
				// We don't need to do a similar check for control panel - u can't uninstall
				// control panel!!
				aMenuPane->SetItemDimmed(cmdQFileMan, EFalse);
			}
							
			// This is faster than NUM_SHORTCUTS calls to FindApp() as long as NUM_SHORTCUTS is less
			// than the number of apps u have installed. However, this is bulkier in code.
			RApaLsSession mySession;
			TApaAppCapabilityBuf aCapabilityBuf;
			
			mySession.Connect();
			mySession.GetAllApps();
			while (mySession.GetNextApp(aInfo) == KErrNone)
			{
				if (mySession.GetAppCapability(aCapabilityBuf, aInfo.iUid) == KErrNone)
				{
					// Only scan applications that are not hidden
					if (aCapabilityBuf().iAppIsHidden == EFalse)
					{
						for (int i = 0; i < NUM_SHORTCUTS; i++)
						{
							if (aInfo.iUid == iTaskView->configData.shortCutUid[i])
							{
								aMenuPane->SetItemDimmed(cmdShort1 + i,EFalse);
								aMenuPane->SetItemTextL(cmdShort1 + i, aInfo.iCaption);
							}
						}
					}
				}
			}
			mySession.Close();
	/*
			// This is the slower but more compact way of doing it
			for (int i = 0; i < NUM_SHORTCUTS; i++)
			{
				if (FindAppFast(iTaskView->configData.shortCutUid[i], &appName))
				{
					aMenuPane->SetItemDimmed(cmdShort1 + i,EFalse);
					aMenuPane->SetItemTextL(cmdShort1 + i, appName);
				}
			}
	*/
		}
	}
}

/*************************************************************
*
* Task view object
*
**************************************************************/

CSMan2TaskListView::~CSMan2TaskListView()
{
	if (taskRefresher->IsActive())
		taskRefresher->Cancel();
	delete taskRefresher;
	
	delete snapshotTasks;
	delete memStatus;
	delete cTaskListBox;
	delete oScanner;
	delete bluejackOne;
	delete flushButton;
	delete endTaskButton;
	delete refreshButton;	
	delete snapshotButton;
	delete compHeapButton;
}

void CSMan2TaskListView::ToggleHotKey(void)
{
	if (iCapturedHotkeyHandle != 0)
		CEikonEnv::Static()->RootWin().CancelCaptureKey(iCapturedHotkeyHandle);
	iCapturedHotkeyHandle = 0;
	if (configData.hotkeyEnabled)
	{
		if ((configData.iHotkeyDialogResult != 0) && (configData.iHotkeyDialogResult != 3)) // 3 = flip open
			iCapturedHotkeyHandle = CEikonEnv::Static()->RootWin().CaptureKey(sCaptureKeyCodes[configData.iHotkeyDialogResult],0,0,2);
	}
}

int CSMan2TaskListView::loadSnapShot(void)
{
	RFile inStream;
	int retVal;
	
	retVal = 1;
	snapshotTasks->Reset();
	if (inStream.Open(iEikonEnv->Static()->FsSession(), snapshotFileName, EFileRead) == KErrNone)
	{
		TInt fileSize;
		
		inStream.Size(fileSize);
		if (fileSize > 1)
		{
			TFileText snapshotFile;
			TBuf<128> buffer;
			
			snapshotFile.Set(inStream);
			while (snapshotFile.Read(buffer) == KErrNone)
				snapshotTasks->AppendL(buffer);
			retVal = 0;
		}
		inStream.Close();
	}
	return retVal;
}

int CSMan2TaskListView::saveSnapShot(void)
{
	RFile outStream;
	int retVal;
	
	retVal = 1;
	if (outStream.Replace(iEikonEnv->Static()->FsSession(), snapshotFileName, EFileRead) == KErrNone)
	{
		TFileText snapshotFile;
		
		snapshotFile.Set(outStream);
		for (int i = 0; i < snapshotTasks->Count(); i++)
		{
			snapshotFile.Write(snapshotTasks->At(i));
		}
		retVal = 0;
		outStream.Close();
	}
	return retVal;
}

int CSMan2TaskListView::loadConfig(void)
{
	RFile inStream;
	TInt retVal;
	
	retVal = 1;
	if (inStream.Open(iEikonEnv->Static()->FsSession(), configFileName, EFileRead) == KErrNone)
	{
		TInt fileSize;
		
		inStream.Size(fileSize);
		if (fileSize >= (TInt)sizeof(verData))
		{
			CVerInfo verData;
			
			// Check config version. If correct, load it else ignore it
			
			TPckg<CVerInfo> pkgVerData(verData);
			inStream.Read(pkgVerData);
			if ((verData.majorVersion == VER_MAJOR) && (verData.minorVersion == VER_MINOR))
			{
				TPckg<CConfig> pkgConfigData(configData);
				inStream.Read(pkgConfigData);
				retVal = 0;
			}
		}
		inStream.Close();
	}
	return retVal;
}

int CSMan2TaskListView::saveConfig(void)
{
	RFile outStream;
	TInt retVal;
	
	retVal = 1;
	if (outStream.Replace(iEikonEnv->Static()->FsSession(), configFileName, EFileRead) == KErrNone)
	{
		verData.majorVersion = VER_MAJOR;
		verData.minorVersion = VER_MINOR;
		TPckg<CVerInfo> pkgVerData(verData);
		outStream.Write(pkgVerData);
		TPckg<CConfig> pkgConfigData(configData);
		outStream.Write(pkgConfigData);
		retVal = 0;
		outStream.Close();
	}
	return retVal;
}

TKeyResponse CSMan2TaskListView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	if (aType == EEventKey)
	{
		if (aKeyEvent.iCode == EQuartzKeyConfirm)
		{
			refreshTaskList(TODO_FOCUS);
			return EKeyWasConsumed;
		}
		else if (aKeyEvent.iCode == EQuartzKeyTwoWayDown)
		{
			cTaskListBox->View()->MoveCursorL(CListBoxView::ECursorNextItem, CListBoxView::ENoSelection);
			cTaskListBox->UpdateScrollBarsL();
			return EKeyWasConsumed;
		}
		else if (aKeyEvent.iCode == EQuartzKeyTwoWayUp)
		{
			cTaskListBox->View()->MoveCursorL(CListBoxView::ECursorPreviousItem, CListBoxView::ENoSelection);
			cTaskListBox->UpdateScrollBarsL();
			return EKeyWasConsumed;
		}
	}
	return cTaskListBox->OfferKeyEventL(aKeyEvent, aType);
}

int CSMan2TaskListView::CountComponentControls() const
{
	return 7;
}

CCoeControl* CSMan2TaskListView::ComponentControl(int aIndex) const
{
	if (aIndex == 0)
		return cTaskListBox;
	else if (aIndex == 1)
		return memStatus;
	else if (aIndex == 2)
		return flushButton;
	else if (aIndex == 3)
		return endTaskButton;
	else if (aIndex == 4)
		return refreshButton;
	else if (aIndex == 5)
		return snapshotButton;
	else if (aIndex == 6)
		return compHeapButton;
	else
		return NULL;
}

void CSMan2TaskListView::ConstructL(const TRect& aRect)
{
	TFileName appPath;
	
	CreateWindowL();
	SetRect(aRect);
	snapshotTasks = new (ELeave) CArrayFixFlat<TApaAppCaption>(3);
	snapshotFileName.Copy(CEikonEnv::Static()->EikAppUi()->Application()->AppFullName());

	appPath.Copy(snapshotFileName);
	appPath.SetLength(snapshotFileName.Length() - 8);  // 8 = "SMAN.APP"
	oScanner = new (ELeave) CScanner(&appPath);

	bluejackOne = new (ELeave) CBlueBeam();
	bluejackOne->logHelper->controlId = cBluejackLog;
	bluejackOne->logBuffer = &(oScanner->logBuffer);

	snapshotFileName.SetLength(snapshotFileName.Length() - 3);
	configFileName.Copy(snapshotFileName);
	snapshotFileName.Append(_L("snp"));
	configFileName.Append(_L("cfg"));

	isFocused = ETrue;

	if (loadSnapShot() == 1)
	{
		iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_NOPREVSNAPSHOT);
	}
	else iEikonEnv->InfoMsgWithAlignment(TGulAlignment(EHRightVCenter), R_TBUF_SNAPSHOTLOADED);

	// Initialize listbox
	cTaskListBox = new(ELeave) CEikColumnListBox;
	cTaskListBox->ConstructL(this, CEikColumnListBox::EMultipleSelection);
	CColumnListBoxData* listBoxColumns = ((CColumnListBoxItemDrawer*)cTaskListBox->ItemDrawer())->ColumnData();
	listBoxColumns->SetColumnWidthPixelL(0, int(Size().iWidth * 0.55));
	listBoxColumns->SetColumnWidthPixelL(1, int(Size().iWidth * 0.32));
	listBoxColumns->SetColumnAlignmentL(0, CGraphicsContext::ELeft);
	listBoxColumns->SetColumnAlignmentL(1, CGraphicsContext::ERight);
	cTaskListBox->CreateScrollBarFrameL();
	cTaskListBox->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto);
	cTaskListBox->SetExtent(TPoint(0, 0), TSize(208, 215));

	// Initialize label
	memStatus = new(ELeave)	CEikLabel;
	memStatusText.Format(_L("      SMan - Free RAM: 0KB"));
	memStatus->SetTextL(memStatusText);
	memStatus->SetContainerWindowL(*this);
	memStatus->SetExtent(TPoint(0, 241), TSize(208, 14));

	TFileName bitmapFile;
	TInt horizOffset, vertOffset;

	horizOffset = 4;
	vertOffset = 7;
	bitmapFile.Copy(appPath);
	bitmapFile.Append(_L("SMAN.MBM"));
	
	// Compress heap button
	compHeapButton = new (ELeave) CEikBitmapButton();
	compHeapButton->SetPictureFromFileL(bitmapFile, 0, 1, CEikCommandButtonBase::EFirst);
	compHeapButton->SetContainerWindowL(*this);
	compHeapButton->SetObserver(this);
	compHeapButton->SetExtent(TPoint(160 + horizOffset, 208 + vertOffset),TSize(40, 26));

	// End task button
	endTaskButton = new (ELeave) CEikBitmapButton();
	endTaskButton->SetPictureFromFileL(bitmapFile, 2, 3, CEikCommandButtonBase::EFirst);
	endTaskButton->SetContainerWindowL(*this);
	endTaskButton->SetObserver(this);
	endTaskButton->SetExtent(TPoint(40 + horizOffset, 208 + vertOffset),TSize(40, 26));

	// Flush button
	flushButton = new (ELeave) CEikBitmapButton();
	flushButton->SetPictureFromFileL(bitmapFile, 4, 5, CEikCommandButtonBase::EFirst);
	flushButton->SetContainerWindowL(*this);
	flushButton->SetObserver(this);
	flushButton->SetExtent(TPoint(0 + horizOffset, 208 + vertOffset),TSize(40, 26));

	// Refresh button
	refreshButton = new (ELeave) CEikBitmapButton();
	refreshButton->SetPictureFromFileL(bitmapFile, 6, 7, CEikCommandButtonBase::EFirst);
	refreshButton->SetContainerWindowL(*this);
	refreshButton->SetObserver(this);
	refreshButton->SetExtent(TPoint(80 + horizOffset, 208 + vertOffset),TSize(40, 26));

	// Snapshot button
	snapshotButton = new (ELeave) CEikBitmapButton();
	snapshotButton->SetPictureFromFileL(bitmapFile, 8, 9, CEikCommandButtonBase::EFirst);
	snapshotButton->SetContainerWindowL(*this);
	snapshotButton->SetObserver(this);
	snapshotButton->SetExtent(TPoint(120 + horizOffset, 208 + vertOffset),TSize(40, 26));

	// Initialize shortcut arrays. This array gets reset by loadConfig() if there is
	// a valid config file to load. So it must be initialised prior to loadConfig()
	for (int i = 0; i < NUM_SHORTCUTS; i++)
	{
		configData.shortCutUid[i] = KNullUid;
	}
	configData.autoSave = CONFIG_AUTOSAVE;
	configData.hotkeyEnabled = CONFIG_HOTKEY_ENABLED;
	configData.resolveName = CONFIG_RESOLVE_HOSTNAME;
	configData.showBluejackSysMsg = CONFIG_SHOW_BLUEJACK_SYSMSG;
	configData.proxSweepOnce = CONFIG_PROX_SWEEP_ONCE;
	configData.flipFlush = CONFIG_FLIPFLUSH;
	configData.forceEndTask = CONFIG_FORCE_ENDTASK;
	configData.noDevicePause = CONFIG_NO_DEVICE_PAUSE;
	configData.noDeviceAttempts = CONFIG_NO_DEVICE_ATTEMPTS;
	configData.showHiddenTasks = CONFIG_SHOW_HIDDEN_TASKS;
	configData.decayTime = CONFIG_DECAY_TIME;
	configData.logFlags = CLogger::logAll;
	configData.obexTimeout = CONFIG_OBEX_TIMEOUT;
	
	// Load config and initialize vars based on config
	loadConfig();
	ToggleHotKey();

	taskRefresher = new (ELeave) CRefreshTaskList;
	
	qFilemanUID.iUid = QFILEMAN_UID;
	cPanelUID.iUid = CONTROLPANEL_UID;
	refreshTaskList(TODO_REFRESH);
	this->EnableDragEvents();
	ActivateL();
}

void CSMan2TaskListView::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	TBool consumed = EFalse;
	TInt theItem;
	TBool pointValid = cTaskListBox->View()->XYPosToItemIndex(aPointerEvent.iPosition, theItem);
	
	if (aPointerEvent.iType == TPointerEvent::EDrag)
	{
		if ((pointValid) && (oldItem != theItem))
		{
			consumed = ETrue;
			cTaskListBox->View()->VerticalMoveToItemL(theItem, CListBoxView::EDisjointSelection);
			cTaskListBox->UpdateScrollBarsL();
			oldItem = theItem;
		}
	}
	else if (aPointerEvent.iType == TPointerEvent::EButton1Down)
	{
		if (pointValid)
		{
			oldItem = theItem;
			consumed = ETrue;
			if (aPointerEvent.iPosition.iX > cTaskListBox->View()->ItemDrawer()->MarkColumn())
			{
				cTaskListBox->ClearSelection();
				cTaskListBox->SetCurrentItemIndex(theItem);
				refreshTaskList(TODO_FOCUS);
			}
			else
			{
				cTaskListBox->View()->VerticalMoveToItemL(theItem, CListBoxView::EDisjointSelection);
				cTaskListBox->UpdateScrollBarsL();
			}
		}
	}
	if (!consumed)
		CCoeControl::HandlePointerEventL(aPointerEvent);
}

void CSMan2TaskListView::refreshTaskList(TInt intTodo)
{
	RApaLsSession oApaLsSession;
	CApaWindowGroupName* oWindowGroupName;
	CArrayFixFlat<TInt>* windowGroupList;
	TInt currentItemIndex;
	CDesCArray *itemArray;
	CArrayFixFlat<TApaAppCaption>* markedArrayText;
	TPtrC currentTaskCaption, temp;
	TApaAppCaption highlightedTask;
	RThread currentTaskThread;
	TBool taskRemoved;
	TThreadId threadId;
	TBool setAppUiFlag = ETrue;
	
	// Set our status as busy
	CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
	if (tempUiObj->isAppUiBusy)
		setAppUiFlag = EFalse;
	else
		tempUiObj->isAppUiBusy = ETrue;

	currentItemIndex = cTaskListBox->CurrentItemIndex();
	itemArray = ((CDesCArray *) cTaskListBox->Model()->ItemTextArray());
	
	// Build the marked array. This is used for TODO_TERMINATE only

	if (intTodo == TODO_TERMINATE)
	{	
		markedArrayText = new CArrayFixFlat<TApaAppCaption>(3);
		for (int i = 0; i < cTaskListBox->SelectionIndexes()->Count(); i++)
		{
			temp.Set(cTaskListBox->Model()->ItemText(cTaskListBox->SelectionIndexes()->At(i)));
			TextUtils::ColumnText(currentTaskCaption, 0, &temp);
			markedArrayText->AppendL(currentTaskCaption);
		}
	}
	else
		markedArrayText = NULL;
	
	// Get the currently highlighted item. Used for focusing to a task
	
	if (intTodo == TODO_FOCUS)
	{
		temp.Set(cTaskListBox->Model()->ItemText(currentItemIndex));
		TextUtils::ColumnText(currentTaskCaption, 0, &temp);
		highlightedTask.Copy(currentTaskCaption);
	}
	
	// Cycle through the list of tasks and rebuild the elements in the listbox
	
	itemArray->Reset();
	cTaskListBox->HandleItemRemovalL();
	windowGroupList = new CArrayFixFlat<TInt>(5);
	iEikonEnv->WsSession().WindowGroupList(windowGroupList);
	oWindowGroupName = CApaWindowGroupName::NewL(iEikonEnv->WsSession());
	oApaLsSession.Connect();
	for (int posIndex = 0; posIndex < windowGroupList->Count(); posIndex++)
	{
		// Check whether this is a task we should display
		oWindowGroupName->ConstructFromWgIdL(windowGroupList->At(posIndex));
		// If we are doing a snapshot the behaviour is:
		// - if hidden tasks is turned off, assume all hidden tasks are checked by default
		// - if hidden tasks is turned on, explicitly follow what the user checked
		// else just follow the hidden task flag as normal
		if ((!configData.showHiddenTasks) && (intTodo != TODO_SNAPSHOT))
		{
			if ((oWindowGroupName->Caption().Length() == 0) || (oWindowGroupName->IsSystem()) || (oWindowGroupName->Hidden()))
				continue;
		}
		else
		{
			if (oWindowGroupName->Caption().Length() == 0)
				continue;
		}
		CApaMaskedBitmap *b = CApaMaskedBitmap::NewLC();
	    if (oApaLsSession.GetAppIcon(oWindowGroupName->AppUid(),TSize(128,128),*b) != KErrNone)
	    {
	    	CleanupStack::PopAndDestroy();
	    	continue;
	    }
    	CleanupStack::PopAndDestroy(); // CApaMaskedBitmap
    	iEikonEnv->WsSession().GetWindowGroupClientThreadId(windowGroupList->At(posIndex), threadId);
    	currentTaskThread.Open(threadId);
    	if (currentTaskThread.Protected())
    	{
    		currentTaskThread.Close();
    		continue;
    	}
    	currentTaskThread.Close();
    	
    	// Now we can do what we were called for
    	taskRemoved = EFalse;
    	currentTaskCaption.Set(oWindowGroupName->Caption());
    	if (intTodo == TODO_FOCUS)
    	{
    		if (currentTaskCaption.Compare(highlightedTask) == 0)
    		{
	    		User::InfoPrint(currentTaskCaption);
				iEikonEnv->WsSession().SetWindowGroupOrdinalPosition(windowGroupList->At(posIndex), 0);
				intTodo = TODO_NOTHING;
    		}
    	}
    	else if (intTodo == TODO_TERMINATE)
    	{
	    	TKeyArrayFix keyArray = TKeyArrayFix(0, ECmpNormal);
	    	int foundIndex;
    		if (markedArrayText->Find(currentTaskCaption, keyArray, foundIndex) == 0)
    		{
				if (!configData.forceEndTask)
					gentleEndTask(currentTaskCaption);
				else
				{
					currentTaskThread.Open(threadId);
					currentTaskThread.Kill(0);
					currentTaskThread.Close();
					taskRemoved = ETrue;
					markedArrayText->Delete(foundIndex);
				}
    		}
    	}
    	else if (intTodo == TODO_SNAPSHOT)
    	{
			snapshotTasks->AppendL(currentTaskCaption);
			if (!configData.showHiddenTasks)
			{
				if (oWindowGroupName->IsSystem() || oWindowGroupName->Hidden())
					taskRemoved = ETrue;
			}
		}
		else if (intTodo == TODO_FLUSH)
		{
	    	TKeyArrayFix keyArray = TKeyArrayFix(0, ECmpNormal);
	    	int foundIndex;

			// Search for the task in the snapshot. If the task wasn't found, kill it
			if (snapshotTasks->Find(currentTaskCaption, keyArray, foundIndex) != 0)
			{
				if (!configData.forceEndTask)
					gentleEndTask(currentTaskCaption);
				else
				{						
					currentTaskThread.Open(threadId);
					currentTaskThread.Kill(0);
					currentTaskThread.Close();
					taskRemoved = ETrue;
				}
			}			
		}
		
		// Done! Now, if the task was still alive we calculate the amount of RAM it
		// is consuming.
		if (!taskRemoved)
		{
			// Weird problem here. I couldn't call GetMemoryInfo for an RProcess. Kept
			// getting KErrNotSupported errors. Calling the deprecated GetRamSizes gives
			// me a panic. Only GetRamSizes for an RThread works. So I have to walk 
			// through all the threads and see which process id it belongs to and add
			// up the RAM utilization based on the pid that I want.

			RProcess targetProcess, currentProcess;
			TFullName procName;
			RThread currentThread;
			TInt iRamSize;
			TBuf<10> procSizeInText;
			TApaAppCaption tempText;

			currentTaskThread.Open(threadId);
			currentTaskThread.Process(targetProcess);
			iRamSize = 0;
			TFindThread findThread;
			while(findThread.Next(procName) == KErrNone)
			{
				currentThread.Open(findThread);
				currentThread.Process(currentProcess);
				if (currentProcess.Id() == targetProcess.Id())
				{
					TInt a, b;
					currentThread.GetRamSizes(a, b);
					iRamSize = iRamSize + a + b;
				}
				currentProcess.Close();
				currentThread.Close();
			}
			currentTaskThread.Close();
			iRamSize = iRamSize / 1024;
			tempText.Copy(currentTaskCaption);
			tempText.Append(KColumnListSeparator);
			procSizeInText.Num(iRamSize);
			tempText.Append(procSizeInText);
			tempText.Append(_L("k"));
			itemArray->AppendL(tempText);				
		}
	}
	oApaLsSession.Close();
	delete windowGroupList;
	delete oWindowGroupName;
	if (markedArrayText != NULL)
		delete markedArrayText;
	
	// Update amount of free memory in system
	int freeRAM;
	HAL::Get(HALData::EMemoryRAMFree, freeRAM);
	memStatusText.Format(_L("      SMan - Free RAM: %dKB"), freeRAM / 1024);
	memStatus->MakeVisible(EFalse);
	memStatus->SetTextL(memStatusText);
	memStatus->MakeVisible(ETrue);

	// Move the highlight back to the item that it was originally at but only if this is
	// a sane value
	// Grab this pointer again because I'm not sure if HandleItemAdditionL() may have
	// moved the address of the array
	itemArray = ((CDesCArray *) cTaskListBox->Model()->ItemTextArray());	
	if (currentItemIndex > (itemArray->MdcaCount() - 1))
		currentItemIndex = itemArray->MdcaCount() - 1;
	// This may be -1 if there were no items in the list box (i.e. we just started up this
	// program)
	if (currentItemIndex < 0)
		currentItemIndex = 0;
	cTaskListBox->SetCurrentItemIndex(currentItemIndex);
	cTaskListBox->HandleItemAdditionL();
	cTaskListBox->UpdateScrollBarsL();
	cTaskListBox->SetFocus(ETrue, ENoDrawNow);
	cTaskListBox->DrawDeferred();

	if (configData.forceEndTask == EFalse)
	{
		if ((intTodo == TODO_TERMINATE) || (intTodo == TODO_FLUSH))
		{
			if (taskRefresher->IsActive())
				taskRefresher->Cancel();
			taskRefresher->startRefresh(REFRESH_DELAY_IF_NOT_FORCE_ENDTASK);
		}
		else if (intTodo == TODO_REFRESH)
		{
			if (taskRefresher->IsActive())
				taskRefresher->Cancel();		
		}
	}

	if (setAppUiFlag)
		tempUiObj->isAppUiBusy = EFalse;
}

void CSMan2TaskListView::gentleEndTask(TPtrC currentCaptionText)
{
	TApaTaskList taskList(iEikonEnv->WsSession());
	TApaTask oneTask = taskList.FindApp(currentCaptionText);
	if (oneTask.Exists())
		oneTask.EndTask();
}

void CSMan2TaskListView::HandleControlEventL(CCoeControl* aControl, TCoeEvent /*aEventType*/)
{
	if (Index(aControl) == 2)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdFlushOut);
	else if (Index(aControl) == 3)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdEndTask);
	else if (Index(aControl) == 4)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdRefresh);
	else if (Index(aControl) == 5)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdSnapshot);
	else if (Index(aControl) == 6)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdCompressHeap);
}

void CSMan2TaskListView::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	gc.Clear(aRect);
}

/*************************************************************
*
* File browser view. This is used by the bluebeam object
*
**************************************************************/

CSMan2FileListView::~CSMan2FileListView()
{
	delete blueBeam;
	delete cFileListBox;
	delete fileList;
	delete labelCurrentFolder;
	delete labelFileDateTime;
	delete labelSmanStatus;
	delete buttonSelectAll;
	delete buttonDeselectAll;
	delete buttonCancel;
	delete buttonSend;
	if (currentPathScroller->IsActive())
		currentPathScroller->Cancel();
	delete currentPathScroller;
}

TKeyResponse CSMan2FileListView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	if (aType == EEventKey)
	{
		if (aKeyEvent.iCode == EQuartzKeyConfirm)
		{
			TPointerEvent fakeEvent;
			
			fakeEvent.iPosition = cFileListBox->View()->ItemPos(cFileListBox->CurrentItemIndex());
			fakeEvent.iPosition.iX = 22;	// Force x coordinate to the right.
			fakeEvent.iType = TPointerEvent::EButton1Down;
			// Note, this event must never be passed to the default pointer event handler!!
			// In this case, we're sure it will never happen because our overridden handler
			// will consume it.
			HandlePointerEventL(fakeEvent);
			return EKeyWasConsumed;
		}
		else if (aKeyEvent.iCode == EQuartzKeyTwoWayDown)
		{
			cFileListBox->View()->MoveCursorL(CListBoxView::ECursorNextItem, CListBoxView::ENoSelection);
			cFileListBox->UpdateScrollBarsL();
			updateFileDateTime();
			return EKeyWasConsumed;
		}
		else if (aKeyEvent.iCode == EQuartzKeyTwoWayUp)
		{
			cFileListBox->View()->MoveCursorL(CListBoxView::ECursorPreviousItem, CListBoxView::ENoSelection);
			cFileListBox->UpdateScrollBarsL();
			updateFileDateTime();
			return EKeyWasConsumed;
		}
	}
	return cFileListBox->OfferKeyEventL(aKeyEvent, aType);
}

void CSMan2FileListView::HandleControlEventL(CCoeControl* aControl, TCoeEvent /*aEventType*/)
{
	if (Index(aControl) == 4)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdBluebeam_SelectAll);
	else if (Index(aControl) == 5)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdBluebeam_DeselectAll);
	else if (Index(aControl) == 6)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdBluebeam_Send);
	else if (Index(aControl) == 7)
		iEikonEnv->EikAppUi()->HandleCommandL(cmdBluebeam_Cancel);
}

void CSMan2FileListView::updateFileDateTime(void)
{
	TTime fileDateTime;

	textFileDateTime = _L("");
	if (fileList->Count() > 0)
	{
		// Compensate for ".." entry. fileList does not have this entry while cFileListBox
		// does!
		if (currentPath.Length() > 3)
		{
			if ((cFileListBox->CurrentItemIndex() > 0) && (cFileListBox->CurrentItemIndex() <= fileList->Count()))
			{
				fileDateTime = (*fileList)[cFileListBox->CurrentItemIndex() - 1].iModified;
				fileDateTime.FormatL(textFileDateTime, _L("%/0%1%/1%2%/2%3%/3 %-B%:0%J%:1%T%:2%S%:3%+B"));
			}
		}
		else
		{
			if (cFileListBox->CurrentItemIndex() <= fileList->Count() - 1)
			{
				fileDateTime = (*fileList)[cFileListBox->CurrentItemIndex()].iModified;
				fileDateTime.FormatL(textFileDateTime, _L("%/0%1%/1%2%/2%3%/3 %-B%:0%J%:1%T%:2%S%:3%+B"));
			}
		}
	}
	labelFileDateTime->SetTextL(textFileDateTime);
	labelFileDateTime->DrawNow();
}

void CSMan2FileListView::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	TBool consumed = EFalse;
	TInt theItem;
	TBool pointValid = cFileListBox->View()->XYPosToItemIndex(aPointerEvent.iPosition, theItem);

	if (cFileListBox->Model()->ItemTextArray()->MdcaCount() > 0)
	{
		if (aPointerEvent.iType == TPointerEvent::EDrag)
		{
			if ((pointValid) && (oldItem != theItem))
			{
				TPtrC tempText, selectedDir;					

				consumed = ETrue;
				tempText.Set(cFileListBox->Model()->ItemTextArray()->MdcaPoint(theItem));
				TextUtils::ColumnText(selectedDir, 1, &tempText);
				if (selectedDir.Compare(_L("<dir>")) != 0)
				{
					cFileListBox->View()->VerticalMoveToItemL(theItem, CListBoxView::EDisjointSelection);
					cFileListBox->UpdateScrollBarsL();
					
				}
				oldItem = theItem;					
			}
		} 
		else if (aPointerEvent.iType == TPointerEvent::EButton1Down)
		{
			if (pointValid)
			{
				TPtrC tempText, selectedDir;					

				consumed = ETrue;
				oldItem = theItem;
				if (aPointerEvent.iPosition.iX > cFileListBox->View()->ItemDrawer()->MarkColumn())
				{
					// Change folder
					tempText.Set(cFileListBox->Model()->ItemTextArray()->MdcaPoint(theItem));
					TextUtils::ColumnText(selectedDir, 1, &tempText);
					
					// Check if user selected directory
					if (selectedDir.Compare(_L("<dir>")) == 0)
					{
						cFileListBox->ClearSelection();				
						// Very briefly, highlight the item
						cFileListBox->SetCurrentItemIndexAndDraw(theItem);
						cFileListBox->SetFocus(ETrue, EDrawNow);
						User::After(5000);
						cFileListBox->Reset();
						
						TextUtils::ColumnText(selectedDir, 0, &tempText);				
						if (selectedDir.Compare(_L("..")) == 0)
						{
							// One level up
							currentPath.SetLength(currentPath.Length() - 1); // remove slash
							currentPath.SetLength(currentPath.LocateReverse(TChar('\\')));
						}
						else
						{
							if (selectedDir.Right(1) != _L(":"))
								currentPath.Append(selectedDir);
							else
								currentPath.Copy(selectedDir);
						}
						currentPath.Append(_L("\\"));
						fillListBox(currentPath);
					}
					else
					{
						cFileListBox->View()->VerticalMoveToItemL(theItem, CListBoxView::EDisjointSelection);
						cFileListBox->UpdateScrollBarsL();
						updateFileDateTime();
					}					
				}
				else
				{
					tempText.Set(cFileListBox->Model()->ItemTextArray()->MdcaPoint(theItem));
					TextUtils::ColumnText(selectedDir, 1, &tempText);
					
					// Only allow the user to select if it is not a dir
					if (selectedDir.Compare(_L("<dir>")) != 0)
					{
						cFileListBox->View()->VerticalMoveToItemL(theItem, CListBoxView::EDisjointSelection);
						cFileListBox->UpdateScrollBarsL();
						updateFileDateTime();
					}
					else
						User::InfoPrint(_L("Can't select a folder."));
				}
			}
			// Prevent user from clicking on the label buttons we use as display for
			// folder path and file date time
			if (aPointerEvent.iPosition.iY < 40)
				consumed = ETrue;
		}
	}
	if (!consumed)
		CCoeControl::HandlePointerEventL(aPointerEvent);
}

void CSMan2FileListView::fillListBox(TPath thePath)
{
	RFs fileServer;
	TBuf<sizeof(TPath)> temp;
	TBuf<20> fileSize;
    CDesCArray *iArray;
	
	temp.Copy(thePath);
	temp.Append(_L("*"));
	fileServer.Connect();
	cFileListBox->ClearSelection();
	iArray = ((CDesCArray *) cFileListBox->Model()->ItemTextArray());
	iArray->Reset();
	cFileListBox->HandleItemRemovalL();
	currentPath.SetLength(0);
	if (fileList)
		delete fileList;
	if (fileServer.GetDir(temp, KEntryAttMaskSupported, ESortByName | EDirsFirst, fileList) == KErrNone)
	{		
		TFileName tempText;
		
		// Navigation upwards. Only allow if thePath is not a root folder
		if (thePath.Length() > 3)
		{
			tempText.Copy(_L(".."));
			tempText.Append(KColumnListSeparator);
			tempText.Append(_L("<dir>"));
			iArray->AppendL(tempText);
		}
		currentPath.Append(thePath);
		for (int i = 0; i < fileList->Count(); i++)
		{
			tempText = (*fileList)[i].iName;
			tempText.Append(KColumnListSeparator);
			
			// Dir check
			if ((*fileList)[i].IsDir())
				tempText.Append(_L("<dir>"));
			else
			{
				if (((*fileList)[i].iSize) < 1024)
				{
					fileSize.Num((*fileList)[i].iSize);
					fileSize.Append(_L("b"));
				}
				else
				{
					fileSize.Num((*fileList)[i].iSize / 1024);
					fileSize.Append(_L("k"));
				}
				tempText.Append(fileSize);
			}
			iArray->AppendL(tempText);
		}
		
		// Always add drive letters
		tempText.Copy(_L("A:"));
		tempText.Append(KColumnListSeparator);
		tempText.Append(_L("<dir>"));
		iArray->AppendL(tempText);
		
		tempText.Copy(_L("C:"));
		tempText.Append(KColumnListSeparator);
		tempText.Append(_L("<dir>"));
		iArray->AppendL(tempText);
		
		tempText.Copy(_L("D:"));
		tempText.Append(KColumnListSeparator);
		tempText.Append(_L("<dir>"));
		iArray->AppendL(tempText);
		
		tempText.Copy(_L("Z:"));
		tempText.Append(KColumnListSeparator);
		tempText.Append(_L("<dir>"));
		iArray->AppendL(tempText);
	}
	else
	{
		User::InfoPrint(_L("Error reading folder."));
		labelFileDateTime->SetTextL(_L(""));
		labelFileDateTime->DrawNow();
	}
	cFileListBox->HandleItemAdditionL();
	if (iArray->MdcaCount() > 0)
	{
		cFileListBox->SetCurrentItemIndexAndDraw(0);
		cFileListBox->SetFocus(ETrue, EDrawNow);
		updateFileDateTime();
	}
	fileServer.Close();
	labelCurrentFolder->SetTextL(currentPath);
	labelCurrentFolder->DrawNow();
}

void CSMan2FileListView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRect(aRect);

	blueBeam = new (ELeave) CBlueBeam();
	blueBeam->logBuffer = &beamLog;
	fileList = NULL;
	
	// Initialize listbox
	cFileListBox = new(ELeave) CEikColumnListBox;
	cFileListBox->ConstructL(this, CEikColumnListBox::EMultipleSelection);
	CColumnListBoxData* listBoxColumns = ((CColumnListBoxItemDrawer*)cFileListBox->ItemDrawer())->ColumnData();
	listBoxColumns->SetColumnWidthPixelL(0, int(Size().iWidth * 0.58));
	listBoxColumns->SetColumnWidthPixelL(1, int(Size().iWidth * 0.30));
	listBoxColumns->SetColumnAlignmentL(0, CGraphicsContext::ELeft);
	listBoxColumns->SetColumnAlignmentL(1, CGraphicsContext::ERight);
	cFileListBox->CreateScrollBarFrameL();
	cFileListBox->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto);
	cFileListBox->SetExtent(TPoint(0, 44), TSize(208, 175));

	// Initialize current folder label
	labelCurrentFolder = new (ELeave) CEikTextButton;
	labelCurrentFolder->SetTextL(currentPath);
	labelCurrentFolder->SetContainerWindowL(*this);
	labelCurrentFolder->SetExtent(TPoint(0, 0), TSize(208, 22));
	labelCurrentFolder->SetBorder(TGulBorder::EDeepSunken);
	labelCurrentFolder->Label()->SetEmphasis(CEikLabel::EFullEmphasis);
		
	// Initialize file date time
	labelFileDateTime = new (ELeave) CEikTextButton;
	textFileDateTime.Copy(_L(""));
	labelFileDateTime->SetTextL(textFileDateTime);
	labelFileDateTime->SetContainerWindowL(*this);
	labelFileDateTime->SetExtent(TPoint(0, 22), TSize(208, 22));
	labelFileDateTime->SetBorder(TGulBorder::ESingleDotted);
	labelCurrentFolder->Label()->SetEmphasis(CEikLabel::ENoEmphasis);
		
	// Initialize sman status
	labelSmanStatus = new (ELeave) CEikLabel;
	labelSmanStatus->SetTextL(_L("            SMan - Bluebeam"));
	labelSmanStatus->SetContainerWindowL(*this);
	labelSmanStatus->SetExtent(TPoint(0, 241), TSize(208, 14));
	
	// Initialize mark all button
	buttonSelectAll = new (ELeave) CEikTextButton;
	buttonSelectAll->SetTextL(_L("All"));
	buttonSelectAll->SetContainerWindowL(*this);
	buttonSelectAll->SetExtent(TPoint(0, 219), TSize(52, 22));
	buttonSelectAll->SetObserver(this);

	// Initialize clear all button
	buttonDeselectAll = new (ELeave) CEikTextButton;
	buttonDeselectAll->SetTextL(_L("None"));
	buttonDeselectAll->SetContainerWindowL(*this);
	buttonDeselectAll->SetExtent(TPoint(52, 219), TSize(52, 22));
	buttonDeselectAll->SetObserver(this);
	
	// Initialize send button
	buttonSend = new (ELeave) CEikTextButton;
	buttonSend->SetTextL(_L("Send"));
	buttonSend->SetContainerWindowL(*this);
	buttonSend->SetExtent(TPoint(104, 219), TSize(52, 22));
	buttonSend->SetObserver(this);

	// Initialize cancel button

	TFileName bitmapFile;
	bitmapFile.Copy(CEikonEnv::Static()->EikAppUi()->Application()->AppFullName());
	bitmapFile.SetLength(bitmapFile.Length() - 3);
	bitmapFile.Append(_L("MBM"));
	
	buttonCancel = new (ELeave) CEikBitmapButton();
	buttonCancel->SetPictureFromFileL(bitmapFile, 10, 11, CEikCommandButtonBase::EFirst);
	buttonCancel->SetContainerWindowL(*this);
	buttonCancel->SetExtent(TPoint(156, 219), TSize(52, 22));
	buttonCancel->SetObserver(this);

	currentPathScroller = new (ELeave) CMarquee;

	// Populate the list box with files...
	fillListBox(_L("C:\\"));
	
	//currentPathScroller->startMarquee(labelCurrentFolder, &currentPath);
	this->EnableDragEvents();
	
	// This stuff is cut and pasted right out of the SDK!
	TZoomFactor zoomFactor;
	zoomFactor.SetZoomFactor(500);
	const CFont* normalFont = iEikonEnv->NormalFont();
	TFontSpec fontSpec = normalFont->FontSpecInTwips();
	if(!zoomFactor.GraphicsDeviceMap())
		zoomFactor.SetGraphicsDeviceMap(iEikonEnv->ScreenDevice()); 
	CFont* font;
    User::LeaveIfError(zoomFactor.GetNearestFontInTwips(font, fontSpec));
    CTextListItemDrawer* itemDrawer = ((CTextListItemDrawer*)cFileListBox->View()->ItemDrawer());
    itemDrawer->SetFont(font);
    cFileListBox->SetItemHeightL(font->HeightInPixels() + cFileListBox->VerticalInterItemGap());
	cFileListBox->View()->ItemDrawer()->SetMarkGutter(0);
	cFileListBox->View()->ItemDrawer()->SetMarkColumnWidth(10);
	ActivateL();
}

int CSMan2FileListView::CountComponentControls() const
{
	return 8;
}

CCoeControl* CSMan2FileListView::ComponentControl(int aIndex) const
{
	if (aIndex == 0)
		return cFileListBox;
	else if (aIndex == 1)
		return labelCurrentFolder;
	else if (aIndex == 2)
		return labelFileDateTime;
	else if (aIndex == 3)
		return labelSmanStatus;
	else if (aIndex == 4)
		return buttonSelectAll;
	else if (aIndex == 5)
		return buttonDeselectAll;
	else if (aIndex == 6)
		return buttonSend;
	else if (aIndex == 7)
		return buttonCancel;
	else
		return NULL;
}

void CSMan2FileListView::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	gc.Clear(aRect);
}

/*************************************************************
*
* Task refresher class
*
**************************************************************/

CRefreshTaskList::CRefreshTaskList() : CTimer(EPriorityNormal)
{
	CTimer::ConstructL();
	CActiveScheduler::Add(this);
}

void CRefreshTaskList::RunL()
{
	TTime currentTime;
	TTimeIntervalSeconds interval;
	
	currentTime.HomeTime();
	if (currentTime.SecondsFrom(startTime, interval) != 0)
		interval = CONFIG_NO_DEVICE_PAUSE + 1;
	if (interval.Int() >= localDelay)
	{
		CSMan2AppUi *tempUiObj = STATIC_CAST(CSMan2AppUi*, CEikonEnv::Static()->EikAppUi());
		tempUiObj->iTaskView->refreshTaskList(TODO_REFRESH);
	}
	else
		After(800000);
}

void CRefreshTaskList::startRefresh(TInt delay)
{
/*
	TBuf<35> refreshMsg;
	
	refreshMsg.Format(_L("Auto refreshing in %d seconds"), delay + 1);
	User::InfoPrint(refreshMsg);
*/	
	localDelay = delay;
	startTime.HomeTime();
	After(800000);
}

/*************************************************************
*
* Marquee class
*
**************************************************************/

CMarquee::CMarquee() : CTimer(EPriorityNormal)
{
	CTimer::ConstructL();
	CActiveScheduler::Add(this);
}

void CMarquee::RunL()
{
	TPtrC *tempText;
	TBuf<sizeof(TPath) + 8> tempPath;
	
	tempText = (TPtrC*)(localLabel->Label()->Text());
	if (localText->Length() > 20)
	{
		tempPath.Copy(tempText->Mid(1, tempText->Length() - 1));
		if (tempText->Length() == localText->Length())
			tempPath.Append(_L("        "));
		tempPath.Append(tempText->Left(1));
		localLabel->SetTextL(tempPath);
		localLabel->DrawNow();
	}
	After(400000);
}

void CMarquee::startMarquee(CEikTextButton *marqueeLabel, TPath *scrollText)
{
	localLabel = marqueeLabel;
	localText = scrollText;
	After(400000);
}

/*************************************************************
*
* External interface
*
**************************************************************/

TUid CSMan2Application::AppDllUid() const
{
	return KUidSMan2App;
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CSMan2Application;
}
