//#include <e32uid.h>		// KExecutableImageUid
//#include <eikdll.h>		// EikDll::StartAppL
#include <gulutil.h>		// TextUtils::ColumnText
#include <eikdoc.h>			// CEikDocument
#include <eikclb.h>			// CEikColumnListBox
#include <eikapp.h>			// CEikApplication
#include <eikappui.h>		// CEikAppUi
#include <eiklabel.h>		// CEikLabel
#include <eikclbd.h>		// CEikColumnListBoxData
#include <eiktxlbm.h>		// CEikTextListBoxModel
#include <apgtask.h>		// TApaTaskList
#include <eikmenub.h>		// CEikMenuBar
#include <apgicnfl.h>		// CApaMaskedBitmap
#include <apgcli.h>			// RApaLsSession
#include <hal.h>			// hardware abstraction layer
#include <hal_data.h>		// memory stats
#include <quartzkeys.h>		// EQuartzKey....Confirm, TwoWayDown, TwoWayUp
#include <apgwgnam.h>		// CApaWindowGroupName
#include <eikcmbut.h>		// CEikBitmapButton
#include <qbtselectdlg.h>	// CQBTUISelectDialog
#include <f32file.h>		// RFs
#include <qikutils.h>		// QikFileUtils
#include "bjack.h"
#include "dialogs.h"

#if !defined(__SMAN_H__)
#define __SMAN_H__

// Actions to perform in AppUi::refreshTaskList()
#define TODO_NOTHING 0
#define TODO_FOCUS 1
#define TODO_TERMINATE 2
#define TODO_SNAPSHOT 3
#define TODO_FLUSH 4
#define TODO_REFRESH 5

// Stuff for the "Start" menu
#define NUM_SHORTCUTS 7
#define QFILEMAN_UID 0x10009e5e
#define CONTROLPANEL_UID 0x100051ff

// Default config values and config file version information
#define VER_MAJOR 1
#define VER_MINOR 1
#define CONFIG_AUTOSAVE ETrue
#define CONFIG_HOTKEY_ENABLED ETrue
#define CONFIG_NO_DEVICE_PAUSE 60
#define CONFIG_NO_DEVICE_ATTEMPTS 3
#define CONFIG_RESOLVE_HOSTNAME ETrue
#define CONFIG_SHOW_BLUEJACK_SYSMSG ETrue
#define CONFIG_PROX_SWEEP_ONCE ETrue
#define CONFIG_FLIPFLUSH EFalse
#define CONFIG_FORCE_ENDTASK EFalse
#define CONFIG_SHOW_HIDDEN_TASKS EFalse
#define CONFIG_DECAY_TIME 30
#define CONFIG_OBEX_TIMEOUT 10

// Misc settings
#define REFRESH_DELAY_IF_NOT_FORCE_ENDTASK 1

const TUid KUidSMan2App = { 0x101FB40F };

static const TUint sCaptureKeyCodes[] = 
{
	0,						// Disable - unused. Place holder so indexing is correct.
	EKeyApplication1,		// Camera
	EKeyApplication0,		// Internet
	0						// Flip open - unused. Place holder so indexing is correct.
};

/*************************************************************
*
* Document class
*
**************************************************************/

class CSMan2Document : public CEikDocument
{
public:
	~CSMan2Document();
	CSMan2Document(CEikApplication& aApp);
	void ConstructL();

private: // from CEikDocument
	CEikAppUi* CreateAppUiL();
};

/*************************************************************
*
* Config class - holds user configurable options
*
**************************************************************/

class CConfig
{
public:
	TInt iHotkeyDialogResult;
	TUid shortCutUid[NUM_SHORTCUTS];
	TBool autoSave;
	TBool hotkeyEnabled;
	TBool resolveName;
	TBool showBluejackSysMsg;
	TBool proxSweepOnce;
	TBool flipFlush;
	TBool forceEndTask;
	TInt noDevicePause;
	TBool showHiddenTasks;
	TInt noDeviceAttempts;
	TInt decayTime;
	TUint logFlags;
	TInt obexTimeout;
};

/*************************************************************
*
* Version of config file
*
**************************************************************/

class CVerInfo
{
public:
	TInt majorVersion;
	TInt minorVersion;
};

/*************************************************************
*
* This class refreshes the task list after a pre-determined
* time. This is used by the gentle termination algorithm
*
**************************************************************/

class CRefreshTaskList : public CTimer
{
public:
	CRefreshTaskList();
	void RunL();
	void startRefresh(TInt delay);
protected:
	TInt localDelay;
 	TTime startTime;
};

/*************************************************************
*
* This is the marquee that will scroll the current folder path
* display in the file browser view
*
**************************************************************/

class CMarquee : public CTimer
{
public:
	CMarquee();
	void RunL();
	void startMarquee(CEikTextButton *marqueeLabel, TPath *scrollText);
protected:
	CEikTextButton *localLabel;
	TPath *localText;
};

/*************************************************************
*
* File browser view. This is the second view
*
**************************************************************/

class CSMan2FileListView : public CCoeControl, public MCoeControlObserver
{
public:
	~CSMan2FileListView();
	TBuf<(MAX_LOG_LINES + 1) * MAX_LOG_LINE_SIZE> beamLog;
	CBlueBeam *blueBeam;
	void ConstructL(const TRect& aRect);
    int CountComponentControls() const;
	CCoeControl* ComponentControl(int aIndex) const;
	void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	CEikColumnListBox *cFileListBox;
	TPath currentPath;
	void fillListBox(TPath thePath);
	TBool dummy;
	CDir *fileList;

	CEikTextButton *labelCurrentFolder;
	TBuf<50> textFileDateTime;
	CEikTextButton *labelFileDateTime;
	CEikLabel *labelSmanStatus;
	CEikTextButton *buttonSelectAll, *buttonDeselectAll, *buttonSend;
	CEikBitmapButton *buttonCancel;

	CMarquee *currentPathScroller;
private:
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	void Draw(const TRect& /*aRect*/) const;
protected:
	TInt oldItem;
	void updateFileDateTime(void);
};

/*************************************************************
*
* Default view. This is also the task list view
*
**************************************************************/

class CSMan2TaskListView : public CCoeControl, public MCoeControlObserver
{
public:
	~CSMan2TaskListView();
	CConfig configData;
	TUid qFilemanUID, cPanelUID;
	TInt32 iCapturedHotkeyHandle;
	CEikColumnListBox *cTaskListBox;
	CArrayFixFlat<TApaAppCaption> *snapshotTasks;
 	TName snapshotFileName, configFileName;
    int CountComponentControls() const;
	CCoeControl* ComponentControl(int aIndex) const;
	void ConstructL(const TRect& aRect);
	void refreshTaskList(TInt intTodo);
	void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	int saveSnapShot(void);
	int saveConfig(void);
	void ToggleHotKey(void);
	CScanner *oScanner;
	CBlueBeam *bluejackOne;

	// Buttons at the bottom
	CEikBitmapButton *flushButton, *endTaskButton, *refreshButton, *snapshotButton,
					 *compHeapButton;

	// Memory status label
	TBuf<50> memStatusText;
	CEikLabel *memStatus;

	// Is SMan in focus?
	TBool isFocused;
private:
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	void Draw(const TRect& /*aRect*/) const;
protected:	
	void gentleEndTask(TPtrC currentCaptionText);
	TInt oldItem;
	CVerInfo verData;
	int loadSnapShot(void);
	int loadConfig(void);
	CRefreshTaskList *taskRefresher;
};

/*************************************************************
*
* App UI class
*
**************************************************************/

class CSMan2AppUi : public CEikAppUi
{
public:
	CSMan2AppUi(){}
	void ConstructL();
	~CSMan2AppUi();
	void HandleForegroundEventL(TBool aForeground);
	void flushTasks(void);
	void HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination);
	// Is SMan's AppUi object busy?
	TBool isAppUiBusy;
	void switchView(TInt viewId);
	CSMan2TaskListView*	iTaskView;
	CSMan2FileListView* iFileView;
protected:
	TInt currentViewType;
	TUint iMenuBar;
	TBool FindApp(TUid appUid, TApaAppInfo *aInfo);
	void AppToForeground(void);
	void DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane);
	void updateSnapShot(void);
	void HandleCommandL(TInt aCommand);
	void LaunchApp(TUid appUid, TFileName docName);
	void sizeToText(TInt64 size, TDes* buffer);
private:
	CCoeControl *iActiveView;
};

class CSMan2Application : public CEikApplication
{
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
};

#endif